/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.acl;

import zero.data.groovy.Manager;

import com.ibm.websphere.acl.PermissionsACL;

import com.ibm.pdq.runtime.data.handlers.ConstantRowHandler;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import zero.core.context.GlobalContext;

import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.acl.ACLRuntimeCache;

def canCreate(collection, user)
{
    return isAuthorized(collection, 0, user, PermissionsACL.CREATE);
}

def canDelete(collection, resource, user)
{
    return isAuthorized(collection, resource, user, PermissionsACL.DELETE);
}

def canRead(collection, resource, user)
{
    return isAuthorized(collection, resource, user, PermissionsACL.READ);
}

def canUpdate(collection, resource, user)
{
    return isAuthorized(collection, resource, user, PermissionsACL.UPDATE);
}

def createAccessControl(collection, resource, user, isGroup = false, permissions = null, checkAccessControl = true)
{
    ACLRuntimeCache.clear(user);

    if ( checkAccessControl && getAccessControl(collection, resource, user, isGroup, false/*cache*/)) {
        throw new Exception(MessageUtils.message('RM12005', [user.toString(), isGroup.toString()]));
    }    
    def acl = [
        resourcename: collection, 
        resourceid: resource, 
        userid: user, 
        isgroup: (isGroup ? 1 : 0), 
        permissions: permissions ?: PermissionsACL.DELETE 
    ];
    
    return createResource('AccessControls', acl, false);
}

def deleteAccessControl(collection, resource, user, isGroup = false)
{
    ACLRuntimeCache.clear(user);

    return deleteAllResources('AccessControls', [resourcename: collection, resourceid: resource, userid:user, isgroup: (isGroup ? 1 : 0)]);
}

def deleteAccessControls(user, isGroup = false)
{
    ACLRuntimeCache.clear(user);

	return deleteAllResources('AccessControls', [userid: user, isgroup: (isGroup ? 1 : 0)]);
}

def deleteAllAccessControls(collection, resource)
{
    ACLRuntimeCache.clear();

	return deleteAllResources('AccessControls', [resourcename: collection, resourceid: resource]);
	
}

def getAccessControl(collection, resource, user, isGroup = false , cache=true)
{
    //logger.INFO { "getAccessControl /${collection}/${resource} and user ${user}... group $isGroup" };
    if (isGroup) {
        return (int)_getAccessControlByGroup(collection, resource, user);
    }

    // check cache first
    if(cache){
	    int cacheddata = ACLRuntimeCache.getPermission(user,collection,resource);
	    if (cacheddata != ACLRuntimeCache.RESULT_UNKNOWN){
	       return cacheddata;
	    }
    }

    // now the expensive way 
    def acl = [
        resourcename: collection, 
        resourceid: resource, 
        userid: user 
    ];
    //logger.INFO { "Looking for ACL for /${collection}/${resource} and user ${user}..." };
	
    def access = null;
   	access = (int) getResourceField('AccessControls', acl, 'permissions', int);
   	//logger.INFO { "User-based ACL lookup result is: ${access}." };
    
   	if (access)
    {
        ACLRuntimeCache.setPermission(user,collection,resource,(int)access);
   		return access;
    }
    	
    //logger.INFO { "getUserGroups( ${user})" };
    def groups =  getUserGroups(user);
        
    groups.each {
         logger.INFO { "_getAccessControlByGroup( ${collection})" };
         def currentPerm = _getAccessControlByGroup(collection, resource, it.id);
         access = (currentPerm > access ? currentPerm : access);
    }
    //logger.INFO { "Group-based ACL lookup result is: ${access}." };
    
    def retvalue = (int) (access ?: PermissionsACL.NONE);
    if(cache){
    	ACLRuntimeCache.setPermission(user,collection,resource,(int)retvalue);
    }
    return retvalue;
}

def _getAccessControlByGroup(collection, resource, groupid)
{
    def acl = [
        resourcename: collection, 
        resourceid: resource, 
        userid: groupid, 
        isgroup: 1
    ];
    def access = getResourceField('AccessControls', acl, 'permissions', int);
    return access ?: PermissionsACL.NONE;
}

def getAccessControlList(collection, resource)
{
    def resources = getResources('AccessControls', [resourcename: collection, resourceid: resource], false);
    return resources;
}

def getAccessControlListNoSystemUsers(collection, resource)
{
    def resources = getAccessControlList(collection, resource);
    // get all system users
	def users = getUsers([systemuser:true]);
	def _systemUserIds = users.id;
	
	// hide all acls that are associated with a system user.
	def _resources = [];
	resources.each { _resource->
		if ( _resource.isgroup ) {
			_resources << _resource;
		}
		else if (!_systemUserIds.contains(_resource.userid)) {
			_resources << _resource;
		}
	}
    
    return _resources;
}

def getAccessLevelKey(value)
{
    return PermissionsACL.NLS_KEYS[value.intValue()];
}

def getResourcesByUserACL(table, filters, user)
{
    def resourceIDs;
//	if ( user == 1 ) {
	if ( isAdmin(user) ) {
		// get all resources
	    resourceIDs = getResources('AccessControls', [resourcename: table], false, ['resourceid']);
	}
	else {
	    resourceIDs = getResources('AccessControls', [resourcename: table, userid: user, isgroup: 0], false, ['resourceid']);
	}
    def groups = getUserGroups(user);
    
    if (groups != null && !groups.isEmpty())
        resourceIDs += getResources('AccessControls', [resourcename: table, userid: groups.id, isgroup: 1], false, ['resourceid']);
    
    if (resourceIDs.isEmpty())
        return [];
    
	filters = filters + [id: resourceIDs.resourceid]
    def resources = getResources(table, filters + [id: resourceIDs.resourceid], false);
    return resources;
}

def isAdmin(user)
{
    // first lookup cache
    int cachedData = ACLRuntimeCache.isAdmin(user);
    if (cachedData != ACLRuntimeCache.RESULT_UNKNOWN)
        return cachedData==ACLRuntimeCache.RESULT_AUTH_SUCCESS;

    // not in cache, so lets lookup the hardway
    def roles = getUserRoles(user);
    boolean isadmin = roles.contains(PermissionsACL.APPLIANCE_ADMIN);
    ACLRuntimeCache.setAdmin(user,isadmin);
    return isadmin;
}

def isAuthorized(collection, resource, user, mode)
{
	// TODO: need to figure out how to fix listing....
    return isAdmin(user) ||  
           getAccessControl(collection, resource, user) >= mode || 
           _isAuthorizedByGroup(collection, resource, user, mode) || checkResource(resource);
}

def checkResource(resource) {
	// this is a work around for the differences between permissions and acls. Allow listing the collection for now
	if ( resource == 0 ) {
	    logger.WARNING { "AccessControls.isAuthorized ${resource} is 0, allow listing" };
	    return true;
	}
	return false;
}

def _isAuthorizedByGroup(collection, resource, user, mode)
{
    def groups =  getUserGroups(user);
    def authorized = groups.find { _getAccessControlByGroup(collection, resource, it.id) >= mode };
    return authorized != null;
}

def _parseID(id)
{
    return id == 'current' ? user.session[].id : id.toInteger();
}

def updateAccessControl(collection, resource, user, isGroup, mode)
{
    ACLRuntimeCache.clear(user);
    def updatedResource = getResource('AccessControls', [resourcename: collection, resourceid: resource, userid: user, isgroup: (isGroup ? 1 : 0)])
    updatedResource['permissions'] = mode;
    
    return updateResource('AccessControls', updatedResource, [] ) ;
    
}

def getResourceField2(resourceType, resourceId, field, type) {
	return getResourceField(resourceType, [id: resourceId], field, type)
}
