/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;
import com.ibm.bedrock.BedrockUtils;
import com.ibm.bedrock.appliance.DNSUtils;

def getDnsServers()
{
	if(BedrockUtils.isBedrock()){
		def dnsServers = DNSUtils.getNameServers();
		def returnValue = [];
		dnsServers.each(){
			def current = [:]
			current['ipaddress'] = it;
			returnValue.add(current);
		}
		return returnValue;
	}else{
		return getDnsServersDeprecated();
	}
	
}
def getDnsServersDeprecated()
{
    def response = executeApplianceRequest([
        name: 'soma:get-status', 
        content: [
            class: 'DNSNameServerStatus'
        ]
    ]);

    if (!response?.status) return []; // if status does not exist, return empty list.
    if (!(response?.status instanceof List || response?.status instanceof Map) ) return [];
    
    def dnsnss = response?.status?.DNSNameServerStatus ?: [];
    
    // Normalize the JSON result in the case of only one result returned.
    if (!(dnsnss instanceof List)) {
    	dnsnss = [dnsnss];    	
    }

    if (dnsnss) {
        dnsnss.each { dns ->
            def keys = dns.keySet().collect { it };

            keys.each { key ->
                dns[key.toLowerCase()] = dns.remove(key);
            }
        }
    }

    return dnsnss;
}
def updateDnsServers(servers)
{
	if(BedrockUtils.isBedrock()){
		def dnsEntries = [];
		servers.each { dns ->
			if(dns && dns.ipaddress){
				dnsEntries.add(dns.ipaddress);
			}
		}
		DNSUtils.setNameServers(dnsEntries as String[]);		
	}else{
		updateDnsServersDeprecated(servers);
	}
}

def updateDnsServersDeprecated(servers)
{
    def subchildren = [];
    
    servers.each { dns -> 
        def server = [
            name: 'NameServers', 
            content: [:],
            children: []
        ];
        
        if (dns.ipaddress)
            server.children << [
                name: 'IPAddress', 
                content: dns.ipaddress
            ];
        
        if (dns.udpport)
            server.children << [
                name: 'UDPPort', 
                content: dns.udpport
            ];
        
        if (dns.tcpport)
            server.children << [
                name: 'TCPPort', 
                content: dns.tcpport
            ];
        
        if (dns.maxretries)
            server.children << [
                name: 'MaxRetries', 
                content: dns.maxretries
            ];
        
        subchildren << server;
    }
    
    def response = executeApplianceRequest([
        name: 'soma:set-config', 
        content: [:], 
        children: [
            [
                name: 'DNSNameService', 
                content: [:], 
                children: subchildren
            ]
        ]
    ]);
            
    if (!response?.result?.equals('OK'))
        return false;
    
    // fix the /etc/resolv.conf file to mirror the DP config
    if (!_writeResolvConf(servers))
        return false;
        
	saveApplianceConfiguration();
    return true;
}

/**
 * Overwrite the existing /etc/resolv.conf file with the proper values on a PUT request. This will make /etc/resolv.conf mirror the datapower config.
 */
def _writeResolvConf (List json) {
	 Iterator it = (Iterator) json.iterator();
	 String resolvLocation = zget('/config/rainmaker/dpResolvConfLocation');
	 
	 File conf = new File (resolvLocation);
	 conf.renameTo(new File(resolvLocation + ".tmp")); // move the file to a temporary location
	 File newConf = new File(resolvLocation);
	 
	 DataOutputStream dos = null;
	 try {
		FileOutputStream fos = new FileOutputStream (newConf);
		dos = new DataOutputStream (fos);		
		
		 while (it.hasNext()) {
			 Object listObj = it.next();
			 String ipAddy = (String) ((Map)listObj).get("ipaddress");// this is a map object
			 dos.writeBytes("nameserver " + ipAddy.trim() + "\n");				 
		 }
		 
		 dos.close();
		 
		 // copy into a temporary location to avoid getting overwritten by firmware upgrades.
		 copyFile (newConf, new File((String)zget('/config/rainmaker/dpResolvConfBackupLocation')).getParentFile());
		 
		 return true;
	} catch (FileNotFoundException e) {
		conf.renameTo(new File(resolvLocation)); // problem, preserve backup.			
	} catch (IOException e) {
		conf.renameTo(new File (resolvLocation)); // problem, preserve backup.
	}		
	finally {
		File backup = new File (resolvLocation + ".tmp");
		backup.delete(); // purge backup copy
	}
	
	return false;
}
