/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import com.ibm.bedrock.appliance.FlashVolume;
import com.ibm.bedrock.appliance.HardwareStatusUtils;
import com.ibm.bedrock.appliance.RaidVolume;
import com.ibm.bedrock.appliance.Sensor;
import com.ibm.bedrock.BedrockUtils;

//
// Supported metric types:
//
// CPUUsage
// EnvironmentalSensors
// FilesystemStatus
// MemoryStatus
// RaidPhysDiskStatus
// RaidVolumeStatus
// SystemUsage
// TemperatureSensors
//

def _lowerCaseMap(map)
{
    def keys = map.keySet().collect { it };
    
    keys.each {
        def value = map.remove(it);
        map[it.toLowerCase()] = value;
    }
    
    return map;
}

def _getMemoryStatus()
{
    if (BedrockUtils.isBedrock()) {
       int totalMem  = HardwareStatusUtils.getTotalMemory ();
       int freeMem = HardwareStatusUtils.getFreeMemory ();

       return [ usedmemory : totalMem - freeMem,
                totalmemory : totalMem ];
    } else {
       System.out.println("Bedrock not detected: Cannot retrieve valid memory status.");
       return [ usedmemory : 0,
                totalmemory : 1 ];
    }
}

def _getFilesystemStatus()
{
    FlashVolume flashStatus = HardwareStatusUtils.getFlashStatus();
    BufferedReader is = new BufferedReader(new InputStreamReader(Runtime.getRuntime().exec("df -m")
            .getInputStream()));
    
    Long encryptCapacity = null;
    Long encryptAvailable = null;
    Long tmpCapacity = null;
    Long tmpAvailable = null;
    
    String line;
    while ((line = is.readLine()) != null) {
        String[] output = line.split("\\s+");
        
        if (output[0].equalsIgnoreCase("tmpfs")) {
            tmpCapacity = Long.valueOf(output[1]);
            tmpAvailable = Long.valueOf(output[3]);
        }
        
        if (output[0].equalsIgnoreCase("/dev/mapper/flash2")) {
            encryptCapacity = Long.valueOf(output[1]);
            encryptAvailable = Long.valueOf(output[3]);
        }
    }
    

    return [
        //totalencrypted2 : encryptCapacity,
        //freeencrypted2 : encryptAvailable,
        totalencrypted : flashStatus.getTotalSize(),
        freeencrypted : flashStatus.getFreeSize(),
        totaltemporary : tmpCapacity,
        freetemporary : tmpAvailable
    ];
}

def _getRaidPhysDiskStatus()
{
	if (!BedrockUtils.isBedrock()) {
		System.out.println("Bedrock not detected: Cannot retrieve valid physical disk status.");
		return [];
	}
	def retobj = [];
    RaidVolume [] volumes = HardwareStatusUtils.getRaidVolumes ();

    long totalSize;
    long totalPartitionFree;
    long totalPartitionSize;
    volumes.each { volume->
        totalSize=0;
        totalPartitionFree=0;
        totalPartitionSize=0;
        totalSize = volume.getTotalSize();
	    
	    // Bedrock can not detect total size for SSD.  For now
	    // remove volumes with size of 0.
	    if (totalSize != 0) {
	        volume.getPartitions().each { partition->
	            totalPartitionSize+=partition.getTotalSize();
	        	totalPartitionFree+=partition.getFreeSize();
	        }
	
	        if (totalPartitionSize != 0) {
	            retobj.add([ 
	                 location : volume.getName(),
	                 role : "primary",
	                 inactivevolume : false,
	                 totalsize : totalPartitionSize,
	                 freesize : totalPartitionFree
	            ]);
	        }
	    }
	}

    BufferedReader is = new BufferedReader(new InputStreamReader(Runtime.getRuntime().exec("df -m")
            .getInputStream()));
    
    Long ndoDiskCapacity = null;
    Long ndoDiskAvailable = null;
    
    String line;
    while ((line = is.readLine()) != null) {
        String[] output = line.split("\\s+");
        
        // 9004
        if (output[0].equalsIgnoreCase("/dev/fioa1")) {
            ndoDiskCapacity = Long.valueOf(output[1]);
            ndoDiskAvailable = Long.valueOf(output[3]);
        }
        // 9005
        if (output[0].equalsIgnoreCase("/dev/md0")) {
            ndoDiskCapacity = Long.valueOf(output[1]);
            ndoDiskAvailable = Long.valueOf(output[3]);
        }
    }
    
    if (ndoDiskCapacity != null && ndoDiskAvailable != null) {
        retobj.add(
             [ 
                 location : "SSD",
                 role : "primary",
                 inactivevolume : false,
                 totalsize : ndoDiskCapacity,
                 freesize : ndoDiskAvailable
             ]);
    }
    
    return retobj;
}

def _getTemperatureSensors()
{
    def retobj = [];
    if (!BedrockUtils.isBedrock()) {
		System.out.println("Bedrock not detected: Cannot retrieve temperature status.");
        return retobj;
    }
    
    Sensor[] sensors = HardwareStatusUtils.getTemperatureStatus(); 

    for( int i = 0; i < sensors.length; i++ ){
        try {
           Float.parseFloat(sensors[i].getReading());
           
           retobj.add(
                [
                    name : sensors[i].getName(),
                    readingstatus : sensors[i].getIsOk(),
                    value : sensors[i].getReading()
                ]);               
        } catch (NumberFormatException nfe) {
            // It's a "discrete" sensor
            String status = "Normal";
            if( (sensors[i].getName().contains("Over-Temperature")) && (sensors[i].getReading() == "true") ){
            	status = "Over";
            } else if( (sensors[i+1].getName().contains("High Temperature")) && (sensors[i+1].getReading() == "true") ) {
            	status = "High";
            }     
              	
            retobj.add(
                [
                    name : sensors[i].getName(),
                    readingstatus : ( (status == "Over") || (status == "High")) ? 'false' : 'true',
                    value : status
                ]);
            i++;            
        }
    }
    return retobj;
}


def getSystemMetrics(type)
{
    def retObj = null;

    if (type.equals("MemoryStatus"))
        retObj = _getMemoryStatus();
    else if (type.equals("FilesystemStatus"))
        retObj = _getFilesystemStatus();
    else if (type.equals("RaidPhysDiskStatus"))
        retObj = _getRaidPhysDiskStatus();
    else if (type.equals("TemperatureSensors"))
        retObj = _getTemperatureSensors();
        

    if (retObj instanceof Map)
        retObj = _lowerCaseMap(retObj);
    else if (retObj instanceof List)
    {
        retObj.each {
            _lowerCaseMap(it);
        }
    }
    
    return retObj;
}
