/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.core;

import zero.core.connection.Connection;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;

import java.util.zip.GZIPInputStream;

/**
 * Digest algorithm equivalents
 */

class FileUtilsStatics
{
    static algorithmTransforms = [ 'SHA1': 'SHA-1' ]
}

List clean(File dir, List extras = [])
{
    def targets = extras;
    
    ['databases', 'export', 'logs', 'reports', 'test-output'].each {
        def next = new File(dir, it);
        
        if (next.exists())
            targets << next;
    }
    
    dir.eachFileMatch(~/rainmaker-.*/) {
        targets << it;
    }
    
    def gcDir = new File(dir, '.zero/private');
    gcDir.eachFileMatch(~/.*zoneStore/) {
        targets << it;
    }
    
    targets.each {
        deleteDir(it);
    }
    
    return targets;
}

void copyDir(File sourceDir, File destinationDir) 
{ 
    destinationDir.mkdirs(); 
     
    sourceDir.eachFile { 
        if (it.isFile()) 
            copyFile(it, destinationDir); 
         
        else 
        { 
            def childDir = new File(destinationDir, it.name); 
            copyDir(it, childDir); 
        } 
    } 
} 
 
void copyFile(File sourceFile, File destinationDir, String newName = sourceFile.name)
{
    logger.FINEST { "copyFile(${sourceFile.absolutePath}, ${destinationDir.absolutePath}, ${newName})" };

    destinationDir.mkdirs();

    def destinationFile = new File(destinationDir, newName);

    def sourceData = sourceFile.newInputStream(); 
     
    destinationFile.withOutputStream {  
        it << sourceData 
    }

    logger.FINEST { "copyFile: copy was successful" };
} 

byte[] copyAndDigestFile(File sourceFile, File destinationDir, String algorithm = 'SHA-1') 
{
    if (FileUtilsStatics.algorithmTransforms[algorithm])
        algorithm = FileUtilsStatics.algorithmTransforms[algorithm]

    destinationDir.mkdirs()

    def sourceIS = new FileInputStream(sourceFile)
    def destOS = new FileOutputStream(new File(destinationDir, sourceFile.name))
    def digestOS = new java.security.DigestOutputStream(destOS, java.security.MessageDigest.getInstance(algorithm))

    byte[] b = new byte[65536]
    for (int len = sourceIS.read(b); len > 0; len = sourceIS.read(b))
        digestOS.write(b, 0, len)

    sourceIS.close()
    digestOS.close()
    destOS.close()

/*
    def destinationFile = new File(destinationDir, sourceFile.name);
    def sourceData = new FileInputStream(sourceFile)

    def digestOS     
    destinationFile.withOutputStream {
        digestOS = new java.security.DigestOutputStream(it,
            java.security.MessageDigest.getInstance(algorithm))

        digestOS << sourceData
        digestOS.close()
    } 
*/

    digestOS.messageDigest.digest()
}


Boolean deleteDir(File dir) 
{
    if (!dir.exists())
        return false;
     
    dir.eachFile { 
        if (it.isFile()) 
            deleteFile(it); 
         
        else 
            deleteDir(it); 
    }
    
    return deleteFile(dir); 
} 
 
Boolean deleteFile(File file) 
{
    logger.FINEST { "deleteFile(${file.absolutePath})"};

    if (!file.exists())
        return false;
     
    def success = file.delete();
 
    if (!success)
    {
        System.gc();
        success = file.delete();
    }
 
    logger.FINEST { "deleteFile: returning ${success}" };
    return success; 
} 


/**
 * Expands the given tar file to the specified directory.
 *
 * @param source object representing tar file, must repsond to withInputStream()
 *   (calls a closure with a java.io.InputStream) and withFile() (calls a closure
 *   with a java.io.File)
 */
def _expand(source, destinationDir, compressed)
{
    // make sure destinationDir exists
    destinationDir.mkdirs(); 
    def rc = 0;


    // Unix
    if (! (System.properties.'os.name' =~ /indows/)) {
        // run tar command
        source.withInputStream() {
            rc = execute([ '/bin/tar', compressed ? 'xzC' : 'xC', destinationDir.path ],
                    // TODO: really should be "null, null, it"
                    false, false, it)
        }
    }


    // Cygwin
    else if (isCygwin()) {
        // run tar command, we use bash -c so tar will run with a
        // reasonable environment
        source.withInputStream() {
            rc = execute([ 'bash', '-c',
                      [ 'tar', compressed ? 'xzC' : 'xC',
                        "'${cygpath(destinationDir.path)}'" ].join(' ') ],
                    // TODO: really should be "null, null, it"
                    false, false, it)
        }
    }
    
    else 
        throw new Exception('Archive expansion cannot be done on this platform.');
    
    return rc;
}


/**
 * Unzips the given zip file to the specified directory.
 *
 * @param source object representing tar file, must repsond to withInputStream()
 *   (calls a closure with a java.io.InputStream) and withFile() (calls a closure
 *   with a java.io.File)
 */
def unzipFile(source, destinationDir, filename='', checkzip=false)
{
    // make sure destinationDir exists
    destinationDir.mkdirs(); 
    def checkFlag = '';

    // Unix
    if (! (System.properties.'os.name' =~ /indows/)) {
        // run tar command
        	if(checkzip)
        	   return execute([ '/usr/bin/unzip', '-t', source.path])
        	else
         	   return execute([ '/usr/bin/unzip', source.path, '-d', destinationDir.path, filename ])
        		
    }


    // Cygwin
    else if (isCygwin()) {
    	if(checkzip)
            return  execute([ 'bash', '-c',
                       [ 'unzip', '-t', " '${cygpath(source.path)}' " ].join(' ') ])
        else
             return  execute([ 'bash', '-c',
                   [ 'unzip', " '${cygpath(source.path)}' ", '-t',
                    "'${cygpath(destinationDir.path)}'", filename ].join(' ') ])
    		 
    }
    
    else 
        throw new Exception('Archive expansion cannot be done on this platform.');
}

/**
 * gunzips the given zip file to the specified directory.
 * Till working on this one.  This will be use on the new OVA that will include 
 * zzip files instead of tgzs for the disks.
 * @param source object representing tar file, must repsond to withInputStream()
 *   (calls a closure with a java.io.InputStream) and withFile() (calls a closure
 *   with a java.io.File)
 */
def expandGzip(source, destinationFile)
{
    source.withInputStream{
    	input->
        def gzipstream = new GZIPInputStream(input);
        destinationFile.withOutputStream{ out -> out <<gzipstream; }
        gzipstream.close()
    }
}

/**
 * Expands a tar file from the specified URL to the destination directory.
 *
 * @param sourceUrl java.net.URL of the tar file
 */
def expandUrl(URL sourceUrl, File destinationDir, Boolean compressed = null)
{
    // guess at compression if not specified
    if (compressed == null)
        compressed = sourceUrl.file.toLowerCase().endsWith('.tgz') ||
                     sourceUrl.file.toLowerCase().endsWith('.tar.gz')

    logger.FINE { "Fetching and expanding ${sourceUrl}" }

    // wrap URL in an Expando
   return _expand(
        new Expando(
            withInputStream: { c ->
                def is = sourceUrl.openStream()
                try { c.call(is) }
                catch (Exception e) { CleanStackTrace.cleanStackTrace(e) }
                finally { is.close() }
            },
            withFile: { c ->
                def f = File.createTempFile('expandUrl',
                                            compressed ? '.tgz' : '.tar')
                f.withOutputStream { os ->
                    def is = sourceUrl.openStream()
                    try { os << is }
                    finally { is.close() }
                }

                try { c.call(f) }
                finally { f.delete() }
            }
        ),
        destinationDir, compressed);
}


/**
 * Expands a tar file from the specified file to the destination directory.
 */
def expandFile(File sourceFile, File destinationDir, Boolean compressed = null)
{
    // guess at compression if not specified
    if (compressed == null)
        compressed = sourceFile.name.toLowerCase().endsWith('.tgz') ||
                     sourceFile.name.toLowerCase().endsWith('.tar.gz')

    // wrap URL in an Expando
    return _expand(
        new Expando(
            withInputStream: { c -> sourceFile.withInputStream(c) },
            withFile: { c -> c.call(sourceFile) }
        ),
        destinationDir, compressed)
}


/**
 * Compresses a directory of files into a single file.
 *
 * @param sourceDir source directory
 * @param destinationFile destination file
 * @param compressed true if destination file should be compressed
 * @param selector closure invoked with each child File to determine if it
 *   should be included
 */
void compressDir(File sourceDir, File destinationFile, Boolean compressed = null, Closure selector = { f -> true })
{
    // guess at compression if not specified
    if (compressed == null)
        compressed = destinationFile.name.toLowerCase().endsWith('.tgz') ||
                     destinationFile.name.toLowerCase().endsWith('.tar.gz')


    // make sure destination parent directory exists
    destinationFile.parentFile.mkdirs(); 


    // Unix
    if (! (System.properties.'os.name' =~ /indows/)) {
        // collect list of files to be tarred
        def files = []
        sourceDir.eachFileRecurse {
            if (! it.isDirectory() && selector.call(it))
                files << relativePath(sourceDir, it)
        }


        // run tar command
        execute([ 'tar', compressed ? 'czfC' : 'cfC',
                  destinationFile.path, sourceDir.path ] + files)
    }


    // Cygwin
    else if (isCygwin()) {
        // collect list of files to be tarred
        def files = []
        sourceDir.eachFileRecurse {
            if (! it.isDirectory() && selector.call(it))
                // paths are all relative so no need for cygpath
                files << relativePath(sourceDir, it).replace('\\', '/')
        }


        // run tar command, we use bash -c so tar will run with a
        // reasonable environment
         execute([ 'bash', '-c',
                  ([ 'tar', compressed ? 'czfC' : 'cfC',
                     "'${cygpath(destinationFile.path)}'",
                     "'${cygpath(sourceDir.path)}'" ] +
                   files.collect { "'${it}'" }).join(' ') ])
    }
    
    else
        throw new Exception('Archive creation cannot be done on this platform.');
}


/**
 * Computes the digest of a file.
 *
 * @param file java.io.File to digest
 * @param algorithm digest algorithm to use
 * @return digest of file as byte[]
 */
byte[] digestFile(File file, String algorithm = 'SHA-1')
{
    if (FileUtilsStatics.algorithmTransforms[algorithm])
        algorithm = FileUtilsStatics.algorithmTransforms[algorithm]

    def dig = new java.security.DigestOutputStream(
        // quick & dirty null output stream implementation
        [ write: { b, off = 0, len = 1 -> } ] as OutputStream, 
        java.security.MessageDigest.getInstance(algorithm)
    )

    file.withInputStream() { dig << it }
    dig.messageDigest.digest()
}


/**
 * Returns the relative path from one file to another.
 *
 * @param from starting file
 * @param to ending file
 * @return relative path (GString)
 */
// TODO - I suspect there are still some edge cases that are wrong in this code
String relativePath(File from, File to)
{
    if (! from.isAbsolute())
        from = from.absoluteFile

    if (! to.isAbsolute())
        to = to.absoluteFile

    def fap = from.absolutePath
    def tap = to.absolutePath

    if (fap == tap)
        return ""

    def fapSlash = fap.endsWith(File.separator) ?
                   fap : "${fap}${File.separator}"
    if (tap.startsWith(fapSlash))
        return tap.substring(fapSlash.length())

    def append = relativePath(from.parentFile, to)
    return append.length() == 0 ? ".." : "..${File.separator}${append}" 
}
