/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.core;

/**
 * Executes a zero task
 *
 * @param task task to be executed (String)
 * @param args arguments for task
 * @param stdout stream to write stdout 
 * @param stderr stream to write stderr
 * @param stdin stream to write stdin
 * @param wait true if i should wait until task is finished before returning
 */
def executeTask(task, args, stdout = null, stderr = null, stdin = null, wait = false)
{
	def cmd = [];	
	
	if(System.getProperty('os.name').startsWith('Windows')) {
		def zerohome = new File(System.getenv("ZERO_HOME"));
		cmd << 'cmd';
		cmd << '/c';
		cmd << zerohome.getCanonicalPath()+File.separator+'zero';
	} else {
		cmd << '/opt/ibm/rainmaker/zeroport';
	}
	cmd << task;
	args.each() {
		cmd << it;
	}
		
	if(wait) {
		execute(cmd);
	} else {
		executeNoWait(cmd);
	}	
}

/**
 * Executes a command.
 *
 * @param cmd command to be executed (String[])
 * @param stdout closure to be invoked with each line of stdout
 * @param stderr closure to be invoked with each line of stderr
 * @return exit code of command
 */
def execute(cmd, stdout = null, stderr = null, stdin = null)
{
    if (! stdout)
        stdout = { info "[${cmd[0]} stdout] + ${it}" }

    if (! stderr)
        stderr = { warning "[${cmd[0]} stderr] ${it}" }

    logger.FINER { "executing command: ${cmd}" }
    def p = cmd.execute()

    def stdoutThread = Thread.start { p.in.eachLine(stdout) }
    def stderrThread = Thread.start { p.err.eachLine(stderr) }

    if (stdin instanceof Closure) {
        p.out << stdin.call()
        p.out.close()
    }
    else if (stdin) {
        p.out << stdin
        p.out.close()
    }
	
    p.waitFor()
    stdoutThread.join()
    stderrThread.join()

    return p.exitValue()
}

/**
 * Executes a command.
 *
 * @param cmd command to be executed (String[])
 * @param stdout closure to be invoked with each line of stdout
 * @param stderr closure to be invoked with each line of stderr
 * @return Process object
 */
def executeNoWait(cmd, stdout = null, stderr = null, stdin = null)
{
    if (! stdout)
        stdout = { [info] "[${cmd[0]} stdout] + ${it}" }

    if (! stderr)
        stderr = { [warning] "[${cmd[0]} stderr] ${it}" }

    logger.FINER { "executing command: ${cmd}" }
    def p = cmd.execute()

    def stdoutThread = Thread.start { p.in.eachLine(stdout) }
    def stderrThread = Thread.start { p.err.eachLine(stderr) }

    if (stdin instanceof Closure) {
        p.out << stdin.call()
        p.out.close()
    }
    else if (stdin) {
        p.out << stdin
        p.out.close()
    }

    p
}
