/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.core;

import zero.data.groovy.Manager;
import zero.data.handler.PagingResultHandler;

import com.ibm.pdq.runtime.data.handlers.ConstantRowHandler;
import com.ibm.pdq.runtime.data.handlers.MapRowHandler;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;

class SQLUtilsStatics
{
    static final INTRANSACTION_NESTING = new ThreadLocal<Integer>();
    
    static startInTransaction()
    {
        def nesting = INTRANSACTION_NESTING.get() ?: 0;
        INTRANSACTION_NESTING.set(++nesting);
        nesting
    }
    
    static endInTransaction()
    {
        INTRANSACTION_NESTING.set(INTRANSACTION_NESTING.get() - 1);
    }
    
    static final MANAGER = new ThreadLocal<Manager>();
    
    static getManager(dbkey, save = false)
    {
        def manager = MANAGER.get();

        if (! manager) {
            manager = Manager.create(dbkey);

            if (save)
                MANAGER.set(manager);
        }

        return manager;
    }
    
    static releaseManager()
    {
        MANAGER.remove();
    }
}

def _getLanguage() {
    return user.lang[] ?: MessageUtils.getLanguageFromLocale(Locale.getDefault());
}

def _createColumnNames(resource, owner, attrFilter)
{
    def columns = [];

    def keepAttr = { true };
    
    if (attrFilter instanceof Closure)
        keepAttr = attrFilter;
    
    else if (attrFilter instanceof Collection)
        keepAttr = { attrFilter.contains(it) };

    if (owner)
        columns << 'ownerid';
    
    resource.each {
        def isID = it.key.equalsIgnoreCase('id');
        
        def validType = _isSchemaType(it.value);

        if (!isID && validType && keepAttr(it.key))
            columns << it.key;
    }
    
    return columns;
}

//
// createResource() adds a row to the specified table using the given resource data. 
// If owner is a valid ID, this method will attempt to set the OwnerID column in the 
// new row of data. If a resource doesn't have an 'owner' or ACL, the last parameter 
// should be false or null.
//
// This method returns 'true' if the resource was created successfully. It also sets 
// the 'id' field of the given resource with the ID generated by the database.
//
def createResource(table, resource, owner, attrFilter = null)
{
    def columns = _createColumnNames(resource, owner, attrFilter);
    columns -= ['created', 'updated'];
    
    def placeholders = ['?'] * (columns.size() + 2);
    
    def values = _createValueList(resource, owner, columns);
    
    columns << 'created' << 'updated';
    def now = System.currentTimeMillis();
    values << now << now;
    
    def db = SQLUtilsStatics.getManager(config.db.key[]);
    def sql = "INSERT INTO ${table} (${columns.join(', ')}) VALUES (${placeholders.join(', ')})";
    
    logger.FINER { "Executing SQL: ${sql}, ${values}" };
    
    try
    {
        resource.id = db.insert(sql, ['ID'], values);
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { MessageUtils.message('RM15009', [t.message]) };
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        
        //
        // HACK: we get the same SQL exception for all database errors, so 
        //       we have to search the actual exception message. this will 
        //       probably break if we ever change database products
        //
        if (t.cause.message =~ /duplicate key/)
            throw new Exception(MessageUtils.message(_getLanguage(), 'RM15001', []), t);
        
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
}


def _createValueList(resource, owner, columns)
{
    def values = [];
    
    columns.each {
        def value = it.equalsIgnoreCase('ownerid') ? owner : resource[it];
        values << (value instanceof GString ? value as String : value);
    }
    
    return values;
}

def _createWhereClauseSQL(filters, owner)
{
    def order = filters.remove('order');
    def desc = filters.remove('descending');
    def exactMatch = filters.remove('exactMatch'); 
    def compare = filters.remove('compare') ?: 'eq';

    def conditions = [];
    def values = [];

    if (owner)
    {
        conditions << 'OwnerID = ?';
        values << owner;
    }
    
    filters.each {
        if (it.value instanceof Number)
        {
            //def operator = _getOperator(compare);
            def operator = it.key =~ /id/ ? '=' : _getOperator(compare);
            conditions << "${it.key} ${operator} ?";
            values << it.value;
        }
        
        else if (it.value instanceof List || it.value instanceof Set)
        {
            conditions << "${it.key} IN (${(['?'] * it.value.size()).join(', ')})";
            values.addAll(it.value);
        }
        
        else if (it.value != null)
        {
            conditions << (exactMatch ? "${it.key} = ?" : "UPPER(${it.key}) LIKE ?");
            values << (exactMatch ? it.value : "%${it.value.toUpperCase()}%".toString());
        }
    }
    
    def text = conditions.isEmpty() ? '' : "WHERE ${conditions.join(' AND ')}";
    
    if (order)
        text += " ORDER BY ${order} ${desc ? 'DESC' : 'ASC'}";
    
    return [
        text: text, 
        values: values 
    ];
}

//
//decrementResourceCounter() atomically retrieves a resource, applies a set
//of changes to it, then updates the resource in the database.  The
//return value is the updated resource if the original resource was found
//in the table, null otherwise.
//
def decrementResourceCounter(table, filters)
{
	 return inTransaction {
         def original = getResource(table, filters, ['counter', 'id']);
         
         if (original)
	     {
		     def updated = [counter: (original.counter - 1), id: original.id];
	         updateResource(table, updated);
	         return true;
	     }
         
         return false;
	 };
}

//
// deleteResource() deletes a row from a table where the ID column 
// matches the value given.
//
// This method returns 'true' if the row was found and deleted.
//
def deleteResource(table, id)
{
    def db = SQLUtilsStatics.getManager(config.db.key[]);

    def sql = "DELETE FROM ${table} WHERE ID = ?";
    
    logger.FINER { "Executing SQL: ${sql}, ${[id]}" };
    
    try
    {
        return inTransaction { db.update(sql, [id]) > 0 };
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { MessageUtils.message('RM15010', [t.message]) };
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
}

//
// deleteAllResources() deletes all rows from a table.
//
// This method returns 'true' if any rows were deleted.
//
def deleteAllResources(table, filters = [:])
{
    def db = SQLUtilsStatics.getManager(config.db.key[]);
    def whereClause = _createWhereClauseSQL(filters, false);
    def sql = "DELETE FROM ${table} ${whereClause.text}";
    
    logger.FINER { "Executing SQL: ${sql}, ${whereClause.values}" };
    
    try
    {
        return inTransaction { db.update(sql, whereClause.values) > 0 };
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { MessageUtils.message('RM15011', [t.message]) };
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
}

//
// getAndDeleteResource() atomically retrieves and deletes a resource
// from a table.  The retrieved resource is returned (natch, otherwise it
// would just be a delete).
//
def getAndDeleteResource(table, filters, columns = [])
{
    return inTransaction {
        def original = getResource(table, filters, columns);

        if (original)
            deleteResource(table, original.id);

        return original;
    };
}

//
// getAndUpdateResource() atomically retrieves a resource, applies a set
// of changes to it, then updates the resource in the database.  The
// return value is the updated resource if the original resource was found
// in the table, null otherwise.
//
def getAndUpdateResource(table, filters, columns = [], updates, updateCol)
{
    return inTransaction {
        def original = getResource(table, filters, columns);
        def updated = original ? original + updates : null;

        if (original)
            updateResource(table, updated, updateCol);

        return updated;
    };
}

//
// getJoinedResources() returns the list of rows matching a given set
// of filters that are also joined via a join table to a particular
// resource.  If the owner parameter is set to a valid ID, the list
// will be trimmed to include those rows where the OwnerID column
// matches the given value.
//
// The 'page' and 'count' filter keys can be used to read through a
// table one 'page' at a time. The 'order' filter key can be used to
// sort the list.
//
def getJoinedResources(table, filters, owner, joinedTable, joinedId, columns = [])
{
    def count = filters.remove('count')?.toInteger();
    def page = filters.remove('page')?.toInteger() ?: 1;

    def db = SQLUtilsStatics.getManager(config.db.key[]);

    // determine join table
    def joinTable = [ table, joinedTable ].sort().join()


    // patch up the supplied filters a bit
    def augmentedFilters = [ "${joinTable}.${singularize(joinedTable)}id": joinedId ];
    filters.each { k, v ->
        if (k == 'order' || k == 'descending')
            augmentedFilters[k] = v;
        else
            augmentedFilters["${table}.${k}"] = v;
    }

    if (owner)
        augmentedFilters["${table}.OwnerID"] = owner;

    // build SQL
    def selectClause = "SELECT ${columns.isEmpty() ? "${table}.*" : columns.collect { "${table}.${it}" }.join(', ')}"

    def fromClause = "FROM ${table} INNER JOIN ${joinTable} ON ${table}.id=${joinTable}.${singularize(table)}id";

    def whereClause = _createWhereClauseSQL(augmentedFilters, false);

    def sql = "${selectClause} ${fromClause} ${whereClause.text}";

    logger.FINER { "Executing SQL: ${sql}, ${whereClause.values}" };
    
    try
    {
        return count ? _getResourcesByPage(db, sql, whereClause.values, count, page) : db.queryList(sql, whereClause.values);
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { MessageUtils.message('RM15012', [t.message]) };
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
}

def _getOperator(name)
{
    switch (name) 
    {
        case 'lt':
            return '<';
        case 'lte':
            return '<=';
        case 'gt':
            return '>';
        case 'gte':
            return '>=';
        case 'neq':
            return '<>';
        case 'eq':
            return '=';
        default:
            logger.WARNING { MessageUtils.message('RM15019', [name]) };
            return '=';
    }
}

//
// getResource() returns the first row of data to match the given set of 
// filters (column-value pairs). If the filter set is empty, it will return 
// the first row in the table. This method returns null if no match was 
// found or the table was empty.
//
def getResource(table, filters, columns = [])
{
    def db = SQLUtilsStatics.getManager(config.db.key[]);
    def whereClause = _createWhereClauseSQL(filters, false);
    def sql = "SELECT ${columns.isEmpty() ? '*' : columns.join(', ')} FROM ${table} ${whereClause.text}";
    
    logger.FINER { "Executing SQL: ${sql}, ${whereClause.values}" };
    
    try
    {
        return db.queryFirst(sql, whereClause.values);
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { MessageUtils.message('RM15013', [t.message]) };
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
}

//
// getResourceCount() returns the number of rows that match the given set of filters.
//
def getResourceCount(table, filters, owner)
{
    def db = SQLUtilsStatics.getManager(config.db.key[]);
    def whereClause = _createWhereClauseSQL(filters, owner);
    def sql = "SELECT COUNT(*) FROM ${table} ${whereClause.text}";
    
    logger.FINER { "Executing SQL: ${sql}, ${whereClause.values}" };
    
    try
    {
        return db.queryFirst(sql, new ConstantRowHandler(int), whereClause.values);
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { MessageUtils.message('RM15014', [t.message]) };
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
}

//
// getResourceField() returns the value of the column in the first row matching the 
// given set of filters. This method returns null if no row matches the filters.
//
def getResourceField(table, filters, field, type)
{
    def db = SQLUtilsStatics.getManager(config.db.key[]);
    def whereClause = _createWhereClauseSQL(filters, false);
    def sql = "SELECT ${field} FROM ${table} ${whereClause.text}";
    
    logger.FINER { "Executing SQL: ${sql}, ${whereClause.values}" };
    
    try
    {
        return db.queryFirst(sql, new ConstantRowHandler(type), whereClause.values);
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { MessageUtils.message('RM15015', [t.message]) };
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
}

//
// getResources() returns the list of rows that match the given set of filters. If 
// the last parameter is set to a valid ID, the list will be trimmed to include 
// those rows where the OwnerID column matches the given value.
//
// The 'page' and 'count' filter keys can be used to read through a table one 'page' 
// at a time. The 'order' filter key can be used to sort the list.
//
def getResources(table, filters, owner, columns = [])
{
    def count = filters.remove('count');
    def page = filters.remove('page');
    
    def db = SQLUtilsStatics.getManager(config.db.key[]);    
    def whereClause = _createWhereClauseSQL(filters, owner);
    def sql = "SELECT ${columns.isEmpty() ? '*' : columns.join(', ')} FROM ${table} ${whereClause.text}";
    
    logger.FINER { "Executing SQL: ${sql}, ${whereClause.values}" };
    
    try
    {
        count = count?.toInteger();
        page = page?.toInteger() ?: 1;
        return count ? _getResourcesByPage(db, sql, whereClause.values, count, page) : db.queryList(sql, whereClause.values);
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { MessageUtils.message('RM15016', [t.message]) };
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
}

def _getResourcesByPage(db, sql, values, count, page)
{
    def results = [];
    
    db.inTransaction {
        def start = (page - 1) * count;
        def handler = new PagingResultHandler(start, count, new MapRowHandler());
        def iterator = db.query(sql, handler, values);
        
        while (iterator.hasNext())
            results << iterator.next();
    }
    
    return results;
}

//
//incrementResourceCounter() atomically retrieves a resource, applies a set
//of changes to it, then updates the resource in the database.  The
//return value is the updated resource if the original resource was found
//in the table, null otherwise.
//
def incrementResourceCounter(table, filters, column = ['counter', 'id'])
{
    return inTransaction {
        def original = getResource(table, filters, column);
        
        if (original)
        {
            def updated = [counter: (original.counter + 1), id: original.id];
            updateResource(table, updated);
            return true;
        }
        
        return false;
    };
}

//
// inTransaction() executes the supplied closure within a single
// database transaction.  inTransaction() returns whatever the closure
// returned.  If the closure throws an exception, inTransaction()
// will throw the same exception.
//
// If the closure throws an exception, inTransaction() rolls back
// the database transaction.  If the closure returns with no exception,
// the database transaction is committed.
//
// Nested to calls to inTransaction() are supported, although all
// of the database calls are presented to the database within a
// single transaction.  This single transaction is not committed or
// rolled back until the closure passed to the outermost call to
// inTransactions() returns or throws an exception.  That is,
// an exception thrown by closures passed to nested invocations of
// inTransaction() will not cause the overall transaction to be
// rolled back unless it propagates out of the closure passed to
// the outermost invocation of inTransaction().
//
def inTransaction(c)
{
    def outerTx = SQLUtilsStatics.startInTransaction() == 1;
    def manager = SQLUtilsStatics.getManager(config.db.key[], outerTx);

    if (outerTx)
        manager.beginTransaction()

    try {
        def result = c()

        if (outerTx)
            manager.commitTransaction()

        return result
    }
    catch (Throwable t) {
        if (outerTx)
            manager.rollbackTransaction()
            
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
    
    finally {
        if (outerTx) {
            manager.endTransaction()
            SQLUtilsStatics.releaseManager();
        }

        SQLUtilsStatics.endInTransaction();
    }
}

def _isSchemaType(value)
{
    switch (value) {
        case null:
        case Boolean:
        case Character:
        case Number:
        case String:
        case GString:
            return true;

        default:
            return false;
    }
}

//
// updateResource() sets the column values in the first row where the ID value 
// is equal to the given resource's 'id' value.
//
// This method returns 'true' if the row was found and updated.
//
def updateResource(table, resource, attrFilter = null)
{
    def changeOwner = false;
    
    if (attrFilter instanceof Collection && attrFilter.remove('ownerid'))
        changeOwner = true;
    
    def columns = _createColumnNames(resource, changeOwner, attrFilter);
    columns -= ['created', 'updated'];
    
    def values = _createValueList(resource, changeOwner, columns);
    
    columns << 'updated';
    values << new Date().getTime();
    
    def db = SQLUtilsStatics.getManager(config.db.key[]);
    
    def sql = "UPDATE ${table} SET ${columns.join(' = ?, ')} = ? WHERE ID = ?";
    
    logger.FINER { "Executing SQL: ${sql}, ${values + resource.id}" };
    
    try
    {
        return db.update(sql, values + resource.id) > 0;
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { MessageUtils.message('RM15017', [t.message]) };
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
}

//
//updateResource() sets the column values in the first row where the ID value 
//is equal to the given resource's 'id' value.
//
//This method returns 'true' if the row was found and updated.
//
def updateResources(table, resource, filters, attrFilter = null)
{
    def columns = _createColumnNames(resource, false, attrFilter);
    columns -= ['created', 'updated'];
    
    def values = _createValueList(resource, false, columns);
    
    columns << 'updated';
    values << System.currentTimeMillis();
    
    def db = SQLUtilsStatics.getManager(config.db.key[]);
    def whereClause = _createWhereClauseSQL(filters, false);
    def sql = "UPDATE ${table} SET ${columns.join(' = ?, ')} = ? ${whereClause}";
    
    logger.FINER { "Executing SQL: ${sql}, ${values}" };
    
    try
    {
        return db.update(sql, values) > 0;
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { MessageUtils.message('RM15018', [t.message]) };
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
        throw new Exception(MessageUtils.message(_getLanguage(), 'RM15000', []), t);
    }
}
