/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
//
// Copyright 2008 Dan Jemiolo
//
// Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software distributed under the License
// is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions and limitations under
// the License.
//

//
// File:     binding.groovy
// Author:   Dan Jemiolo
// License:  Apache Software License 2.0
// URL:      http://codeunleashed.com/2008/02/binding.htm
// Email:    danielj at gmail dot com
//
// Usage:
//
//     $ groovy bindings /path/to/your/app
//
// If no path is provided, the script defaults to your current directory.
//
// Description:
//
// This script reads through the Groovy files in a Zero application's /app/scripts directory
// and creates Groovy bindings for each function it finds. The bindings are provided as Java
// classes - one for each file in /app/scripts. A config file that registers the Groovy bindings
// is created in the application's /config directory, and the zero.config file is modified to
// have an @include statement that points to it. This system allows the tool to generate bindings
// into the application with as little disruption as possible to the existing files.
//
// The code is organized with utility methods first (in alphabetical order), followed by the
// script's entry point; to find the entry point, search for "execution starts here!".
//

//
// capitalize() returns the given string with the first character upper-cased.
//
import com.ibm.websphere.rainmaker.logging.Output;

def capitalize(s)
{
    def first = s[0].toUpperCase();
    return s.length() == 1 ? first : first + s.substring(1);
}

//
// createBindingsClass() reads through the given script and creates a Java class that
// implements Zero's BindingHandler interface and has a binding for each function in
// the script. The generated class is saved under the application's /java directory.
//
def createBindingsClass(scriptFile, javaFile)
{
    //
    // first, find all of the function definitions in the script
    //

    def reader = new BufferedReader(new FileReader(scriptFile));
    def functions = [];
    def next = null;

    while ((next = createFunction(reader)) != null)
        functions << next;

    if (functions.isEmpty())
        return false;

    reader.close();

    Output.info("Generating file ${javaFile.getName()} ...");

    //
    // now fill in the template for the bindings class with
    // the function info
    //

    def javaDir = javaFile.getParentFile();
    javaDir.mkdirs();

    def writer = new FileWriter(javaFile);

    createBindingsHeader(writer, scriptFile, javaFile);
    createBindingsRegistration(writer, functions);

    functions.each() {
        createBindingsMethod(writer, scriptFile, it);
    }

    createBindingsFooter(writer);

    writer.flush();
    writer.close();

    Output.info("Finished writing file ${javaFile.getName()}.");
    return true;
}

//
// createBindingsConfig() adds a config stanza to the given file that adds a set of
// Java class names to the /config/bindings/.groovy property.
//
def createBindingsConfig(configFile, javaFiles)
{
    def writer = new FileWriter(configFile);

    writer.write("\n/config/bindings/.groovy += [\n");

    javaFiles.sort().eachWithIndex() { it, index ->
        writer.write("    \"zero.groovy.bindings.${getClassName(it)}\"");

        if (index < javaFiles.size() - 1)
            writer.write(",");

        writer.write("\n");
    }

    writer.write("]\n");
    writer.flush();
    writer.close();
}

//
// createBindingsInclude() adds an @include statement to the application's zero.config file
// that points to the file with the bindings in it. The @include is only added if it does not
// already exist.
//
def createBindingsInclude(appDir, configFile, bindingsFile)
{
    def include = "@include \"\${/config/dependencies/ibm:${appDir.getName()}}/config/${bindingsFile.getName()}\"".toString();
    def hasInclude = false;

    if (configFile.exists())
    {
        configFile.eachLine() {
            if (!hasInclude && it.trim().equals(include))
                hasInclude = true;
        }
    }

    if (!hasInclude)
    {
        def writer = new FileWriter(configFile, true);
        writer.write("\n${include}\n");
        writer.flush();
        writer.close();
    }
}

//
// createBindingsHeader() generates the comments, package declaration, imports and class declaration
// for the given Java class. It also creates the implementation of the addVariables() method, which
// is used by BindingHandler implementations to declare what functions they are binding to the
// global namespace.
//
def createBindingsHeader(writer, scriptFile, javaFile)
{
    def className = getClassName(javaFile);

    writer.write("""/*
IBM Confidential
OCO Source Materials
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009
The source code for this program is not published or otherwise
divested of its trade secrets, irrespective of what has been
deposited with the U.S. Copyright Office.
*/

//
// ${className} is a Groovy binding handler for ${scriptFile.getName()}.
//
// Please see http://codeunleashed.com/2008/02/binding.htm for more information
// about the tool that generated this code.
//

package zero.groovy.bindings;

import java.io.*;
import java.net.*;
import java.util.*;

import org.codehaus.groovy.runtime.MethodClosure;

import zero.core.groovysupport.bindings.InvokeBindings;

public class ${className} extends InvokeBindings
{
    private static final ${className} SINGLETON = new ${className}();

    private static final Map<String,Object> METHOD_CLOSURES = new java.util.HashMap<String,Object>();
""");
}

def createBindingsFooter(writer)
{
    writer.write("}\n");
}

def createBindingsRegistration(writer, functions)
{
    writer.write("""
    static
    {
""");

    functions.each() {
        writer.write("        METHOD_CLOSURES.put(\"${it.name}\", new MethodClosure(SINGLETON, \"${it.name}\"));\n");
    }

    writer.write("""    }

    public void addVariables(Map<String, Object> variables)
    {
        super.addVariables(variables);
        variables.putAll(METHOD_CLOSURES);
    }
""");
}

//
// createBindingMethod() generates the Java method that finishes the binding process by invoking
// the right script/method combination for a given function call.
//
def createBindingsMethod(writer, scriptFile, function)
{
    def scriptPath = scriptFile.path.tokenize(File.separator)[-4..-1].join('/');

    if (function.optParams)
    {
        writer.write("""
    public ${function.returnType} ${function.name}(${createParameterString(function.paramTypes, function.params, function.optParams)})
        throws FileNotFoundException, NoSuchMethodException
    {
        Object[] _args_ = new Object[${function.params.size()} + rest.length];
""");

        function.params.eachWithIndex() { name, index ->
            writer.write("        _args_[${index}] = ${name};\n");
        }

        writer.write("""
        System.arraycopy(rest, 0, _args_, ${function.params.size()}, rest.length);
        ${function.returnType != 'void' ? 'return (' + function.returnType + ') ' : ' '}invokeMethod(\"${scriptPath}\", \"${function.name}\", _args_);
    }
""");
    }

    else
        writer.write("""
    public ${function.returnType} ${function.name}(${createParameterString(function.paramTypes, function.params, function.optParams)})
        throws FileNotFoundException, NoSuchMethodException
    {
        ${function.returnType != 'void' ? 'return (' + function.returnType + ') ' : ' '}invokeMethod(\"${scriptPath}\", \"${function.name}\", new Object[]{ ${function.params.join(", ")} });
    }
""");
}

//
// createClassName() creates a Java class name for a Groovy script. If the script is named
// myTools.groovy, the class will be MyToolsBindings.
//
def createClassName(scriptFile)
{
    return capitalize(scriptFile.getName().replace(".groovy", "Bindings"));
}

//
// createFunction() reads through the given file to find the next function definition. It returns
// a map with two properties that describe the function: 'name' and 'params'. The former is a string
// and the latter a list of strings. If no definition is found (because we've reached EOF), this
// function returns null.
//
def createFunction(reader)
{
    def nextLine = null;
    def m = null;

    while ((nextLine = reader.readLine()) != null)
    {
        // <return_type> <method_name> '(' <parameters> ')' [ 'throws' <exception>,...] [ '{' ]
        m = nextLine =~ /^([\w\.\$\[\]]+)\s+(\w+)\s*\(([^\)]*)\)\s*(?:throws\s+[\w\s\.\,]+)?\s*\{?\s*$/
        if (! m || m.group(2).startsWith('_'))
            continue;

        break;
    }

    if (nextLine == null)
        return null;

    def returnType = m.group(1) == 'def' ? 'Object' : m.group(1);
    def name = m.group(2);

    def parameterString = m.group(3);
    def parameterList = parameterString.tokenize(",");
    def paramTypes = [];
    def params = [];
    def optParams = 0;

    parameterList.each() {
        // [ <type> ] <name> [ '=' <default_value> ]
        m = it =~ /^\s*(?:([\w\.\$\[\]]+)\s+)?(\w+)\s*(=.*)?\s*$/
        if (m)
            if (m.group(3))
                optParams++;
            else
            {
                paramTypes << (m.group(1) ?: 'Object');
                params << m.group(2);
            }
    };

    Output.info("Found function ${name} with return type ${returnType} and parameters ${params} with types ${paramTypes} ${optParams ? '(+' + optParams + ' optional parameters) ' : ''}...");

    return [
        returnType: returnType,
        name: name,
        paramTypes: paramTypes,
        params: params,
        optParams: optParams
    ];
}

//
// createParameterString() creates a parameter list for a Java method declaration using a list
// of parameter names.
//
def createParameterString(paramTypes, params, optParams)
{
    def x = [];
    params.eachWithIndex() { it, index -> x << (paramTypes[index] + ' ' + it) };
    if (optParams)
        x << "Object... rest";

    return x.join(', ')
}

//
// getClassName() returns the name of the class represented by a Java file.
//
def getClassName(javaFile)
{
    def fileName = javaFile.getName();
    def dot = fileName.lastIndexOf('.');
    return fileName.substring(0, dot);
}

def hasBeenModified(scriptFile, javaFile)
{
    return !javaFile.exists() || scriptFile.lastModified() > javaFile.lastModified();
}

//
// execution starts here!
//

def module = event.apphome[].name.tokenize('.')[-1];

def scriptsDir = new File(event.apphome[], "app/scripts");

def scripts = [];

new File(scriptsDir, "groovy/rainmaker").eachFile { packageDir ->
    if (packageDir.name == '.svn')
        return;

    packageDir.eachFileMatch(~/.*\.groovy/) {
        // don't generate bindings for test scripts
        if (event.apphome[].name.endsWith('.test') && it.name.endsWith('Test.groovy'))
            return;

        scripts << it;
    }
}

if (scripts.size() == 0)
{
    Output.info("There are no scripts in ${scriptsDir.path}");
    return;
}

Output.info("Starting code generation for application ${event.apphome[].name}");

def javaDir = new File(event.apphome[], "java/zero/groovy/bindings");
def javaFiles = [];

scripts.each() {
    Output.info("Looking for functions in ${it.path} ...");

    def javaFile = new File(javaDir, "${createClassName(it)}.java");

    def addToBindingFile = true;

    if (!hasBeenModified(it, javaFile))
        Output.info("Skipping file - it has not been modified since the last generation.");

    else if (!createBindingsClass(it, javaFile))
        addToBindingFile = false;

    if (addToBindingFile)
        javaFiles << javaFile;

    Output.info("Finished looking for functions in ${it.path}.");
}

if (!javaFiles.isEmpty())
{
    Output.info("Adding new bindings to config files ...");

    def bindingsFile = new File(event.apphome[], "config/bindings.config");
    def configFile = new File(event.apphome[], "config/zero.config");

    createBindingsConfig(bindingsFile, javaFiles);
    createBindingsInclude(event.apphome[], configFile, bindingsFile);

    Output.info("Finished updating config files.");
}

Output.info("Finished code generation. You must restart your application for the changes to take effect.");

