/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.rest.RESTUtils;
import com.ibm.websphere.rainmaker.logging.MessageUtils;

def _redact(r) {
    def REDACTED_KEYS = [
        'currentmessage', 'currentstatus', 'email', 'fullname', 'id',
        'username'
    ];

    // attribute mappings
    if (r.containsKey('name'))
        r.fullname = r.name;

    // re-fetch partial objects
    if (! r.keySet().containsAll(REDACTED_KEYS)) {
        r = getUser(r.id);
        r.fullname = r.name;
    }

    // cull attributes
    r.keySet().retainAll(REDACTED_KEYS);

    // add translated attributes
    r.currentmessage_text = r.currentmessage ? MessageUtils.message(user.lang[], r.currentmessage) : null;
    r.currentstatus_text = r.currentstatus ? MessageUtils.message(user.lang[], r.currentstatus) : null;

    // add relationship attributes
    r.groups = getUserGroups(r.id).collect { "/resources/groups/${it.id}" }
    r.patterns = getPatterns([ownerid: r.id]).collect { "/resources/patterns/${it.id}" }
    r.virtualsystems = getInstances([ownerid: r.id]).collect { "/resources/virtualsystems/${it.id}" }

    return r
}


/**
 * @success 200 Returns the list of users defined on the
 * CloudBurst appliance.  See the example below for a sample of the
 * data returned.
 * @error 403 If the requester does not have sufficient permission
 * to view the list of users.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * [
 *    {
 *       "currentmessage": "RM02011",
 *       "currentmessage_text": "Active in the last five minutes",
 *       "currentstatus": "RM01061",
 *       "currentstatus_text": "Logged in and active",
 *       "email": "",
 *       "fullname": "Administrator",
 *       "groups": [
 *          "/resources/groups/1"
 *       ],
 *       "id": 1,
 *       "patterns": [
 *          "/resources/patterns/1",
 *          "/resources/patterns/2",
 *          "/resources/patterns/3",
 *          "/resources/patterns/4",
 *          "/resources/patterns/5",
 *          "/resources/patterns/8"
 *       ],
 *       "username": "xcadmin",
 *       "virtualsystems": [
 *          "/resources/virtualsystems/13",
 *          "/resources/virtualsystems/15"
 *       ]
 *    },
 *    {
 *       "currentmessage": "RM02013",
 *       "currentmessage_text": "User has not logged in yet",
 *       "currentstatus": "RM01062",
 *       "currentstatus_text": "Inactive",
 *       "email": "testuser@mycompany.com",
 *       "fullname": "Test User",
 *       "groups": [
 *          "/resources/groups/1"
 *       ],
 *       "id": 2,
 *       "patterns": [
 *       ],
 *       "username": "testuser",
 *       "virtualsystems": [
 *       ]
 *    }
 * ]
 * </pre>
 *
 * <p>See the description of GET /resources/users/{id} for attribute
 * details.</p>
 */
def onList() 
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = request.json.output[].collect { _redact(it) }
        }]

    RESTUtils.handleOnList([name: request.params.name[], username: request.params.username[]]);
}


/**
 * @success 200 Returns information about a user defined to the
 * WebSphere CloudBurst appliance.
 * @error 403 If the requester does not have permission to view
 * the specified user.
 * @error 404 If the requested user is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * {
 *    "currentmessage": "RM02011",
 *    "currentmessage_text": "Active in the last five minutes",
 *    "currentstatus": "RM01061",
 *    "currentstatus_text": "Logged in and active",
 *    "email": "",
 *    "fullname": "Administrator",
 *    "groups": [
 *       "/resources/groups/1"
 *    ],
 *    "id": 1,
 *    "patterns": [
 *       "/resources/patterns/1",
 *       "/resources/patterns/2",
 *       "/resources/patterns/3",
 *       "/resources/patterns/4",
 *       "/resources/patterns/5",
 *       "/resources/patterns/8"
 *    ],
 *    "username": "xcadmin",
 *    "virtualsystems": [
 *       "/resources/virtualsystems/13",
 *       "/resources/virtualsystems/15"
 *    ]
 * }
 * </pre>
 *
 * <p>A user has the following attributes:</p>
 *
 * <p>currentmessage - Message associated with the current status of
 * the user.  This string value is generated by the appliance.</p>
 *
 * <p>currentmessage_text - Textual representation of currentmessage.
 * This is a string representation of currentmessage in the
 * requester's preferred language and is automatically generated by
 * the appliance.</p>
 *
 * <p>currentstatus - A string constant representing the current
 * status of the user.  This value is automatically generated by the
 * appliance.</p>
 *
 * <p>currentstatus_text - Textual representation of currentstatus.
 * This is a string representation of currentstatus in the
 * requester's preferred language and is automatically generated by
 * the appliance.</p>
 *
 * <p>email - The user's email address.</p>
 *
 * <p>fullname - The user's full name.</p>
 *
 * <p>groups - List of URIs of the groups to which this user belongs.
 * The URIs are relative should be resolved against the URI of the
 * user that contains them.</p>
 *
 * <p>id - Id of the user.  This value is numeric and is automatically
 * generated by the appliance.</p>
 *
 * <p>paterns - List of URIs of the patterns owned by this user.  The
 * URIs are relative should be resolved against the URI of the user
 * that contains them.</p>
 *
 * <p>username - Login name for the user.</p>
 *
 * <p>virtualsystems - List of URIs of the virtual systems owned by
 * this user.  The URIs are relative should be resolved against the
 * URI of the user that contains them.</p>
 *
 * <p>Note that the user's password can be specified when the user is
 * created and subsequently changed, but is never returned as part of
 * the user information.</p>
 */
def onRetrieve()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = _redact(request.json.output[]);
        }]

    RESTUtils.handleOnRetrieve(com.ibm.websphere.rest.RESTUtils.getRequestedUserID());
}


/**
 * @success 201 The user has been defined and is included in the
 * response body.  The URL of the new user is included in the
 * response's Location header.
 * @error 400 If there are problems parsing the JSON data in the
 * request.
 * @error 403 If the requester does not have sufficient permission
 * to define a new user.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <p>Request JSON:</p>
 *
 * <pre>
 * {
 *    "email": "newuser@mycompany.com",
 *    "fullname": "New User",
 *    "password": "newuserpw",
 "    "username": "newuser"
 * }
 * </pre>
 *
 * <p>Response JSON:</p>
 *
 * <pre>
 * {
 *    "currentmessage": "RM02013",
 *    "currentmessage_text": "User has not logged in yet",
 *    "currentstatus": "RM01062",
 *    "currentstatus_text": "Inactive",
 *    "email": "newuser@mycompany.com",
 *    "fullname": "New User",
 *    "groups": [
 *       "/resources/groups/1"
 *    ],
 *    "id": 3,
 *    "patterns": [
 *    ],
 *    "username": "newuser",
 *    "virtualsystems": [
 *    ]
 * }
 * </pre>
 */
def onCreate()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion()) {
        request.filters.in = [{
            if (it.fullname)
                it.name = it.fullname;

            it.remove('fullname');
            return it;
        }];

        request.filters.out = [{
            request.json.output = _redact(request.json.output[]);
        }];
    }

    RESTUtils.handleOnCreate();
}


/**
 * @success 200 The user was successfully updated.  The response body
 * contains a JSON representation of the user's current state.
 * @error 400 If there are problems parsing the JSON data in the
 * request.
 * @error 403 If the requester does not have permission to update
 * the specified user.
 * @error 404 If the request references a resource that is not
 * defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <p>Request JSON:</p>
 *
 * <pre>
 * {
 *    "currentmessage": "RM02013",
 *    "currentmessage_text": "User has not logged in yet",
 *    "currentstatus": "RM01062",
 *    "currentstatus_text": "Inactive",
 *    "email": "changedemail@mycompany.com",
 *    "fullname": "Changed Fullname",
 *    "groups": [
 *       "/resources/groups/1"
 *    ],
 *    "id": 3,
 *    "password": "changedpassword",
 *    "patterns": [
 *    ],
 *    "username": "newuser",
 *    "virtualsystems": [
 *    ]
 * }
 * </pre>
 *
 * <p>Response JSON:</p>
 *
 * <pre>
 * {
 *    "currentmessage": "RM02013",
 *    "currentmessage_text": "User has not logged in yet",
 *    "currentstatus": "RM01062",
 *    "currentstatus_text": "Inactive",
 *    "email": "changedemail@mycompany.com",
 *    "fullname": "Changed Fullname",
 *    "groups": [
 *       "/resources/groups/1"
 *    ],
 *    "id": 3,
 *    "patterns": [
 *    ],
 *    "username": "newuser",
 *    "virtualsystems": [
 *    ]
 * }
 * </pre>
 */
def onUpdate()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion()) {
        request.filters.in = [{
            if (it.fullname)
                it.name = it.fullname;

            it.remove('fullname');
            return it;
        }];

        request.filters.update = [{ user ->
            def newids = user.groups.collect { group ->
                def m = group =~ /\/resources\/groups\/(\d+)/;
                if (m)
                    return m.group(1).toInteger();
            }

            if (! newids.every() ||
                ! newids.contains(1)) {
                logger.FINE { "returning 404 for invalid user.groups: ${user.groups}" };
                com.ibm.websphere.rest.RenderUtils.renderError(404, 'RM08004');
            }

            def oldids = getUserGroups(user.id).collect { it.id };

            (oldids - newids).each { removeGroupMember(it, user.id) }
            (newids - oldids).each { addGroupMember(it, user.id) }

            user.remove('groups');
            return user;
        }];

        request.filters.out = [{
            request.json.output = _redact(request.json.output[]);
        }];
    }

    def id = com.ibm.websphere.rest.RESTUtils.getRequestedUserID();
    
    RESTUtils.handleOnUpdate(id) { resource -> 
        if (!resource.password)
            return;
        
        def session = user.session[];
        
        if (!session.permissions.contains(Permissions.APPLIANCE_ADMIN))
            throw new Exception(MessageUtils.message(user.lang[], 'RM08008', []));
            
        def userName = getResourceField('users', [id: id], 'username', String);
        
        logger.INFO { MessageUtils.message('RM08070', [session.username, userName]) };
    
        changePassword(resource.id, resource.password);
        
        logger.INFO { MessageUtils.message('RM08071', [userName]) };
    }
    
    //
    // update user's session data to match any 
    // changes to their account
    //
    if (id == user.session[].id)
    {
        def updatedUser = getUser(id);
        def session = user.session[];
        updatedUser.each {
            session[it.key] = it.value;
        }
        user.session = session;
    }
}


/**
 * @success 204 The user definition has been deleted.
 * @error 403 If the requester does not have permission to delete
 * the user definition.
 * @error 404 If the requested user is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 */
def onDelete()
{
    def id = com.ibm.websphere.rest.RESTUtils.getRequestedUserID();
    
    //
    // HACK: original admin user can never be deleted
    //
    if (id == 1)
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    RESTUtils.handleOnDelete(id) { deleted ->
        invalidateSession(deleted);
    }
}
