/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.zip.ZipOutputStream;
import java.util.zip.ZipEntry;

import com.ibm.websphere.rainmaker.tracing.TraceFileHandler;
import com.ibm.websphere.rainmaker.error.ErrorFileHandler;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;

import zero.json.Json;

import com.ibm.ws.xsa.common.ProblemDeterminationUtils;

def _addContentToZip(zipOut, zipEntryName, content)
{
    def entry = new ZipEntry(zipEntryName);
    zipOut.putNextEntry(entry);
    
    def bytes = content.getBytes();
    zipOut.write(bytes, 0, bytes.length);
}

def _addFileToZip(zipOut, file)
{
	if(!file.isFile()) return;
	
    def buffer = new byte[2048];
    def bytesRead = 0;
    
    file.withInputStream {
        def entry = new ZipEntry(file.name);
        zipOut.putNextEntry(entry);
        
        while ((bytesRead = it.read(buffer, 0, buffer.length)) >= 0)
            zipOut.write(buffer, 0, bytesRead);
    }
}

def _generateErrorReport()
{
    def response = executeApplianceRequest([
        name: 'soma:do-action', 
        content: [:], 
        children: [
            [
                name: 'ErrorReport', 
                content: [:], 
                children: [
                    [
                        name: 'InternalState', 
                        content: 'on'
                    ]
                ]
            ]
        ]
    ]);
    return response?.result?.equals('OK');
}

def onList()
{	
	APPLIANCE_ADMIN = 2; // define this here to get rid of the runtime dependency on r.users
	
	 if (!("xcadmin".equals(request.pdsession[]) || user.session[].permissions.contains(APPLIANCE_ADMIN)))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
	request.headers.out.'Content-Type' = 'application/zip';
    request.headers.out.'Content-Disposition' = 'inline; filename=trace.zip'; 
    request.headers.out.Expires = '0';
    request.headers.out.Pragma = 'cache';
    request.headers.out.'Cache-Control' = 'private';
    
    
    def zipOut = new ZipOutputStream(request.outputStream[]);
    def diskRoot = zget("/config/rainmaker/disk/root")
    if(config.rainmaker.appName[]){ // strip off appName
    	diskRoot = diskRoot.substring(0, (diskRoot.length() -config.rainmaker.appName[].length() ))
    }
	
	// look for special parms to force out dumps at some type of interval, and run jextract on cores
	// dumpProc - alone, will take a single dump of the specified process
	// numDumps - specifies the number of dumps to force, will be ignored if dumpProc is not specified
	// dumpInt - specifies the time in seconds to wait between dumps , will be ignored if dumpProc is not specified.  numDumps will default to 3.
	// deleteDumps - specifies whether to remove any dumps located after collected.
	if(zcontains("/request/params/dumpProcs")) {
		def dumpProcName = zget("/request/params/dumpProcs");
		def numDumps = zget("/request/params/numDumps");
		def dumpInt = zget("/request/params/dumpInt");
		ProblemDeterminationUtils.sendSIGQUITsignal(dumpProcName, numDumps, dumpInt);
	}
	
	
    try
    {
    	// Here we are putting the contents of many folders under diskRoot into the trace.zip file
        def diskRootDir = new File(diskRoot);
        if(diskRootDir.exists()){
        	// diskRootDirChildren is a directory such as ${diskRootDir.path}/cs-start
        	// ${diskRootDir.path}/xsServer00 on the appliance
        	def diskRootDirChildren = diskRootDir.listFiles();
        	def subdirs = ["", "error", "trace", "audit", "ffdc", "0", "1"];
        	diskRootDirChildren.each(){
        		// only worry about sMash applications leveraging CB logfilehandlers for this collection
        		if(it.isDirectory()){
        			// we are no longer making zip files out of the child directories under the diskroot
        			// ex: cs, cs-setup, xsa.app, etc.
        			// they should remain child directories at the root of trace.zip
	        		def logsDir = new File (diskRootDir, it.name + "/logs");
	        		def serverNameFile = logsDir.list(
	        			    [accept:{d, f-> f ==~ /($it.name).*/ }] as FilenameFilter
	        			  )
	        		if(serverNameFile){
	        			subdirs.addAll(serverNameFile.toList());
	        		}
	        		try{
			            // subdirs is a directory like trace.zip/cs/logs/error
			            for (def i=0; i < subdirs.size(); i++){
			            	// logsDir is a directory like ${diskRootDir.path}/xsa.app/logs
			            	// currentLog is a directory like trace.zip/cs/logs/trace or trace.zip/xsa.app/logs/error
			                def currentLog = new File (logsDir, subdirs[i]);
			                if(currentLog.exists()){
			                    currentLog.eachFileMatch(~/.*(log|gz|txt|phd|dmp)/) { file ->
			                    	// "file" is something like trace.zip/xsa.app/logs/trace/trace.log
			                    	def filePath = file.path;
			                    	// logsDir is a directory like ${diskRootDir.path}/xsa.app/logs

			                    	filePath = filePath.substring("${diskRootDir.path}".length()+1 ); //add one to remove last separator
			                    	
			                    	// filePath is a directory like xsa.app/logs/trace/trace.log
    			                	zipOut.putNextEntry(new ZipEntry(filePath));
    			                	new FileInputStream(file).withStream { inStream ->
	    			                	def buffer = new byte[8192]
	    			                	def count;
	    			                	while((count = inStream.read(buffer, 0, 8192)) != -1) {
	    			                		zipOut.write(buffer,0,count)
	    			                	}
				                    }
    			                	zipOut.closeEntry()
			                    }
			                }
			            } // end traversal of subdirs
	        		} catch (Throwable t) {
			                logger.SEVERE { CleanStackTrace.cleanStackTrace(t) }
	        		} // end try
        		} // end diskRootChildren folder traversal
        	}  // end diskRootDirChildren.each()
        }
                
		// in the following sections, we add files directly to the root of trace.zip
    	def strApplianceLogFile = "/drouter/log/appliance.log";
    	def applianceLogFile = new File(strApplianceLogFile); 
    	if (applianceLogFile.exists()){
    		_addFileToZip(zipOut, applianceLogFile);
    	}
		try {
            // zdump('/') no longer works b/c we added a context sMash doesn't know about - _appliance.
            // Work around is to all zdump on all the top level contexts we know about.
			//def dumpData = zdump('/')
            def globalContexts = [
                'appliance',
                'config',
                'request',
                'storage'
                //'tmp' - is a bunch of translated messages, we don't need to dump this.
            ];
            
            for (globalContext in globalContexts) {
                def dumpData = zdump('/'+globalContext);
                if(dumpData!=null) {
                    _addContentToZip(zipOut, 'global-context-'+globalContext+'-snapshot.log', dumpData);  
                }            
            }

		    if(System.getProperty('java.vendor').startsWith('IBM')) {
		    	if(zcontains("/request/params/hdump")) {
	    			com.ibm.jvm.Dump.HeapDump();
			    	def heapDumpDir = System.getenv("IBM_HEAPDUMPDIR");
			    	if(heapDumpDir==null) {
	    				heapDumpDir = diskRoot;
			    	}
			    	if(heapDumpDir != null){
				    	heapDumpDir = new File(heapDumpDir);
				    	heapDumpDir.eachFileMatch(~'heapdump.*') { file ->
				    	    _addFileToZip(zipOut, file);
				    		file.delete();
				    	}
			    	}
			    }
			    
			    if(zcontains("/request/params/jdump")) {
		    		com.ibm.jvm.Dump.JavaDump();
			    }
			    // always grab the javacores
		    	def coreDumpDir = System.getenv("IBM_JAVACOREDIR");
		    	if(coreDumpDir==null) {
    				coreDumpDir = diskRoot;
		    	}
		    	if(coreDumpDir != null){
			    	coreDumpDir = new File(coreDumpDir);
			    	coreDumpDir.eachFileMatch(~'javacore.*') { file ->
			    		_addFileToZip(zipOut, file);
			    	}
		    	}
		    }
		    if(zcontains("/request/params/oscore")) {
		    	def coreDumpDir = diskRoot;
		    	if(coreDumpDir != null){
			    	coreDumpDir = new File(coreDumpDir);
			    	coreDumpDir.eachFileMatch(~'core(|.*)') { file ->
			    		_addFileToZip(zipOut, file);
			    	}
		    	}
		    }
		    
		    if(zcontains("/request/params/diskUsage") ){
		    	diskRootDir = new File(diskRoot);
			    def directorySizeMap = [:];
			    _calculateDirectorySize(diskRootDir, directorySizeMap, diskRootDir);
			    
			    String directorySizeLog = diskRoot + "/logs/directorySizeDump.log";
			    File directorySizeLogFile = new File (directorySizeLog);
			    BufferedWriter bw = new BufferedWriter(new FileWriter(directorySizeLogFile));
			    zero.json.Json.encode(directorySizeMap, bw);
			    
			    _addFileToZip(zipOut, directorySizeLogFile);		    	
	    	}
		    			
		} catch(Throwable e) {
			logger.SEVERE { CleanStackTrace.cleanStackTrace(e) };
		}    

		try {			
			def packetFile = new File('/drouter/temporary/capture.pcap');
			if (packetFile.exists())
				_addFileToZip(zipOut, packetFile);
	        
			def temporaryDirectory = new File('/drouter/temporary');
			if (temporaryDirectory.exists()){
				temporaryDirectory.eachFileMatch(~/.*log/) { file ->
		            if(!file.isDirectory()){
		                _addFileToZip(zipOut, file);
				    }else{
				    	file.eachFileMatch(~/.*log/) { logFile ->
				            if(!logFile.isDirectory())
				                _addFileToZip(zipOut, logFile);
				    	}
				    }
				}
			}
			
			def drouterDir = new File('/drouter');
			def localDirectory = new File('/drouter/local');
			if (localDirectory.exists()){
				localDirectory.eachFileMatch(~/.*log/) { file ->
            	    // "file" is something like /drouter/local/xsa.admin_firstBoot.log
            	    def localDirFilePath = file.path;
            	    localDirFilePath = localDirFilePath.substring("${drouterDir.path}".length() + 1);
            	
            	    // filePath is a directory like local/xsa.admin_firstBoot.log
            	    zipOut.putNextEntry(new ZipEntry(localDirFilePath));
            	
            	    new FileInputStream(file).withStream { inStream ->
                	    def buffer = new byte[2048];
                	    def count;
                	    while((count = inStream.read(buffer, 0, 2048)) != -1) {
                		    zipOut.write(buffer);
                	    } //end writing while loop
                    } // end FileInputStream closure
                	zipOut.closeEntry();
			    } // end localDirectory file-matching closure
			} //end if					
		} catch (Throwable t) {
            logger.SEVERE { CleanStackTrace.cleanStackTrace(t) }
        }

    	try{
    		if(onAppliance()) {
	    		def applianceStatus = executeApplianceRequest([
	    	                                                name: 'soma:get-status', 
	    	                               		        	content: [ : ]
	    	                                              ]);
		    	if(applianceStatus){
		    		applianceStatus = zero.json.Json.encode(applianceStatus);	// convert to string
		    		_addContentToZip(zipOut, 'appliance-get-status-snapshot.log', applianceStatus);
		    	}
    		}
    	} catch (Throwable t){
    		logger.SEVERE { CleanStackTrace.cleanStackTrace(t) }
    	}


	try{
            Runtime.getRuntime().exec("/bin/cli -e platform collect-pd").waitFor();
            def collectPd = new File ('/tmp/userfiles/collect-pd.txt');
            if(collectPd.exists()){
        	_addFileToZip(zipOut, collectPd);
            }
    	} catch (Throwable t){
    		logger.SEVERE { CleanStackTrace.cleanStackTrace(t) }
    	}

	try{
            def initFio = new File ('/tmp/init-fio.log');
            if(initFio.exists()){
        	_addFileToZip(zipOut, initFio);
            }
    	} catch (Throwable t){
    		logger.SEVERE { CleanStackTrace.cleanStackTrace(t) }
    	}
		
        def generatedErrorFile = new File('/drouter/temporary/error-report.txt');
        //
        // if no error report is present, generate a new one
        //
        try
        {
            if (!generatedErrorFile.exists())
    		    _generateErrorReport();
        } catch (Throwable t) {
            logger.SEVERE { CleanStackTrace.cleanStackTrace(t) }
        }
		
		if (generatedErrorFile.exists())
            _addFileToZip(zipOut, generatedErrorFile);

        def varLogDirectory = new File ('/var/log/');
        if(varLogDirectory.exists()){
        	varLogDirectory.eachFileMatch(~/(diag-log|default-trace)(|.*)/) { file ->
            	_addFileToZip(zipOut, file);
            }
        }
	}
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08009', t);
    }
    
    finally
    {
        zipOut.close();
    }
}

def _calculateDirectorySize(dir, directorySizeMap, diskRoot) {
	
	File[] filesInDir = dir.listFiles();
	long sizeOfFiles = 0;
	for (int i = 0; i < filesInDir.length; i++) {
		if (filesInDir[i].isDirectory()) {
			sizeOfFiles += _calculateDirectorySize(filesInDir[i], directorySizeMap, diskRoot);
		}
		else if (filesInDir[i].isFile()) {
			sizeOfFiles += filesInDir[i].length();
		}
	}
	
	// Strip off the /drouter/ramdisk2.../raid0 part of the file name and do not output directories less than 100k
	if (sizeOfFiles >= 100000) {
		directorySizeMap.put(dir.getAbsolutePath().substring(diskRoot.getAbsolutePath().length()), (int)(sizeOfFiles/1024));
	}
	
	return sizeOfFiles;
}
