/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.ui;

import java.text.DateFormat;

import com.ibm.websphere.acl.PermissionsACL;
import groovy.rainmaker.constants.Status;
import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;

def getCachedResource(type, id)
{
    def cacheKey = "/request/cache/${type}/${id}";
    def cached = zget(cacheKey);

    def resource = cached ?: getResource(type, [id: id]);

    if (!cached)
        zput(cacheKey, resource);

    return resource;
}

def renderDateTime(id, timestamp = false)
{
    if (!timestamp)
        timestamp = System.currentTimeMillis();

    def date = new Date(timestamp);
    
    def lang = user.lang[].split('_')[0];
    def country = user.country[];
    def locale = country ? new Locale(lang, country) : new Locale(lang);
    
    def formatter = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, locale);
    return formatter.format(date);
}

def renderExternalField(name, resource, type = false)
{
    request.external.resource = resource;

    if (!type)
        type = request.params.render.resourceType[];

    def template = "external/${type}/${name}.gt";
    
    def file = getVFile("app/views/" + template, null);
    
    if (!file?.exists()) {
    	//logger.INFO { "could not find $template, try external/${name}.gt instead" };
        template = "external/${name}.gt";
    }
    else {
    	//logger.INFO { "found $template" };
    }
    return renderGroovyTemplate(template);
}

/**
 *
 * renderField() inspects the name and type of a field and generates the proper HTML widget
 * to represent it. If the field represents a relationship, it will create one or more links
 * to the resources on a separate page.
 *
 * @param map
 *        The hash map (JSON object) containing the HTTP response data.
 *
 * @param name
 *        The name of the JSON field to render.
 *
 * @param htmlId
 *        The ID to assign to the HTML input element so that it can be retrieved later.
 *
 * @param mutable
 *        True if the field can be edited.
 *
 * @return The HTML for an input element that is appropriate for the given field.
 *
 */
def renderField(map, name, htmlId, mutable) {

    renderField(map, name, htmlId, mutable, "");
}

def renderField(map, name, htmlId, mutable, relatedType)
{
    def session = user.session[];
    // TODO - escape value
    def value = map[name];
    
    // parse out @foo=bar options
    def options = [:];
    def m;
    while ((m = name =~ /(.*)@([^=]+)=([^-@]*)(.*)/).matches()) {
        name = "${m.group(1)}${m.group(4)}";
        options[m.group(2)] = m.group(3);
    }
    //
    // HACK: if resources are in Status.published make mutable = false
    //

    if(map.currentstatus?.equals(Status.PUBLISHED))
    	mutable = false;

    //
    // HACK: handle special formatting of some cloud data
    //

    // Handle Security Certificate display
    if (name.equals('certificate'))
    {
        def certIsAccepted = map.certified == 'T';
        value = certIsAccepted ? 'RM10312' : 'RM10311';
        String shortDesc = MessageUtils.message(user.lang[], value, []);
        String longDesc = MessageUtils.message(user.lang[], value, []);

        def function = null;
        def text = null;
        def link = null;

        if ([Status.FAILED, Status.MAINTENANCE].contains(map.currentstatus))
        {
            function = certIsAccepted ? 'removeCertificate' : 'displayCertificate';
            text = certIsAccepted ? MessageUtils.message(user.lang[], 'RM10009', []) : MessageUtils.message(user.lang[], 'RM10316', []);
        }

        if (function)
            link = "[<a href=\"javascript:document.dashboard.details.${function}('/resources/hypervisors', ${map.id}, '${map.certified}');\">${text}</a>]";

        return "<span class='default'><img src='${icon(value)}' alt=\"${shortDesc}\" title=\"${shortDesc}\" class='icon-image' style='vertical-align: middle;'/>&nbsp;&nbsp;${longDesc}&nbsp;&nbsp;${link ?: ''}</span>";
    }

    // TODO - dead code?
    if (name.equals('ipaddressid'))
        return renderIP(value, htmlId);

    // belongs-to link to another resource
    if (!name.equals('id') && name.endsWith('id') && value instanceof Number)
    {
        // Special rendering for a network subnetid field
        if (name == 'subnetid')
            return renderSubnetSelectBox(name, value, htmlId, mutable);

        else
            return renderBelongsToLink(name, value);
    }

    else if (name.startsWith('enum-'))
    {
        name = name.substring(5);
        return renderEnumeration(name, map[name], htmlId, mutable);
    }

    else if (name.startsWith('external-'))
    {
        name = name.substring(9);
        return renderExternalField(name, map);
    }

    else if (name.startsWith('fileupload-'))
    {
        name = name.substring(11);
        return renderFileUpload(name, map);
    }

    // has-many links to another resource
    else if (name.startsWith('hasmany-'))
    {
    	def type = request.params.render.resourceType[]
    	def id = map.id
    	if(type == 'templates' && name.substring(8) == 'part2s')
    		id = map.currenteditionid;
        return renderHasManyLinks(name.substring(8), id, session.id, htmlId, mutable, options);
    }

    // child-many-many links to another resource
    else if (name.startsWith('childmanymany-'))
        return renderChildManyManyLinks(name.substring(14), map.id, session.id, htmlId, mutable, relatedType);

    // parent-many-many links to another resource
    else if (name.startsWith('parentmanymany-'))
        return renderParentManyManyLinks(name.substring(15), map.id, session.id, htmlId, mutable, relatedType);

    // parent-many-many links to another resource
    else if (name.startsWith('subnetnetwork-'))
        return renderSubnetNetworkLinks(name.substring(14), map.id, session.id, htmlId, mutable, relatedType);

    // many-many links to another resource
    else if (name.startsWith('manymany-'))
    {
    	def type = request.params.render.resourceType[]
    	def id = map.id
    	if(type == 'templates' && name.substring(9) == 'patterns')
    		id = map.currenteditionid;
        return renderManyManyLinks(name.substring(9), id, session.id, htmlId, mutable, options);
    }

    if (value == null)
        value = '';

    if (value instanceof String && MessageUtils.isMessage(value))
    {
        if (name =~ /status/)
            return renderStatus(value, map.currentmessage ?: value, htmlId);

        else
            value = MessageUtils.message(user.lang[], value, []);
    }

    // date/time
    else if (name =~ /created/ || name =~ /updated/)
        return "<span class='default' id='${htmlId}'>${renderDateTime(htmlId, value)}</span>";

    // no-op for strings - guarantees we get a string from the object
    value = value.toString();

    if (value.length() == 0)
        value = MessageUtils.message(user.lang[], 'RM10032', []);

    else if (name.endsWith('password'))
    {
        //
        // the only time we display a password field on the properties panel is on
        // the Users and Hypervisors pages. an admin can always edit these passwords
        //
        // this condition needs to be updated if we ever have more resource types
        // that have a password field displayed and they are not always mutable
        //
        def type = request.params.render.resourceType[];
        def isMutable = type ==~ /users/ ? !storage.rainmaker.security.enabled[] : true;
        def path = type ==~ /users/ ? "${request.path[]}/password" : request.path[];

        request.params.render.uri = path;
        request.params.render.mutable = isMutable;
        return renderGroovyTemplate('widgets/changePassword.gt');
    }

    if (name == 'vendor') {
        return renderHypervisorType(value, htmlId);
    }

    // anything else
    return renderText(value, htmlId);
}

def renderIP(ipId, htmlId)
{
	if(!ipId)
		return renderText("", htmlId);
	def ip = getCachedResource('ips', ipId);

    if (!ip.hostname)
    {
        try
        {
            ip.hostname = InetAddress.getByName(ip.ipaddress).getHostName();
        }

        catch (Throwable t)
        {
            logger.WARNING { CleanStackTrace.cleanStackTrace(t) };
        }
    }

    def text = ip.hostname ? "${ip.ipaddress} (${ip.hostname})" : ip.ipaddress;
    return renderText(text, htmlId);
}

def renderStatus(value, description = null, htmlId = '')
{
    def shortDesc = MessageUtils.message(user.lang[], value, []);
    def longDesc = description ? MessageUtils.message(user.lang[], description, []) : shortDesc;
    
    longDesc = longDesc.replaceAll("\\\"", "\\\\\"")

    def html = new StringBuilder();
    
    html.append("""
        <span class='default' id='${htmlId}'>
        <img src='${icon(value)}' alt=\"${shortDesc}\" title=\"${shortDesc}\" class='icon-image' style='vertical-align: bottom;' onError='\$(this).style.display="none";'/>
    """);
    
    if (description)
        html.append("&nbsp;&nbsp;${longDesc}");
    
    html.append("</span>");
    
    if (htmlId?.length() > 0)
        html.append("""
            <script type='text/javascript'>
                document.dashboard.details.setCurrentStatus("${icon(value)}", "${longDesc}");
            </script>
        """);
    
    return html.toString();
}

def renderHypervisorType(value, htmlId)
{
    def text = "";

    // Combine ESX and ESXi
    if (value.startsWith("ESX")) {
        text = MessageUtils.message(user.lang[], 'RM03000', []);
    } else {
        text = value;
    }
    return "<span class='default' id='${htmlId}'>${text}</span>";
}

def renderText(value, htmlId)
{
    return "<span class='default' id='${htmlId}'>${value}</span>";
}
