/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.users;

import groovy.rainmaker.constants.*;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.rainmaker.auditing.AuditDataHelper;
import com.ibm.websphere.rainmaker.users.RainmakerLdapService;

def addGroupMember(group, user)
{
    if (isUserInGroup(group, user))
        return;

    def moreThanOneGroup = isUserInMoreThanOneGroup(user);
    createResource('GroupsUsers', [groupid: group, userid: user], false);
    // check it is not EveryoneGroup
    if (group != 1) {
        if (!moreThanOneGroup) {
            // clear out old user roles setting first
            getUserRoles(user).each { deleteUserRole(user, it); }
        }
        // set user roles setting based on group roles setting
        getGroupRoles(group).each { addUserRole(user, it); }
    }
	
	
}

def createEveryoneGroup()
{
    def lang = MessageUtils.getLanguageFromLocale(Locale.getDefault());
    
    createGroup([
        name: MessageUtils.message(lang, 'RM02014', []), 
        description: MessageUtils.message(lang, 'RM02015', [])
    ], 1);
}

def createGroup(group, owner)
{
	
    // if LDAP security is enabled, make sure is a valid LDAP group
    if (storage.rainmaker.security.enabled[]) {
        if (!RainmakerLdapService.isValidGroup(group.name)) {
            throw new Exception(MessageUtils.message(_getLanguage(), 'RM02026', []));
        }
    }

	
    if (getGroupByName(group.name))
        throw new Exception(MessageUtils.message(user.lang[] ?: MessageUtils.getLanguageFromLocale(Locale.getDefault()), 'RM02021', [group.name]));
        
    createResource('Groups', group, owner);    

    // Audit hook
    def result = getGroup(group.id);
    if(result != null) {
    	def fieldsToTrans = [];
    	AuditDataHelper.transCodeToMessage(result, fieldsToTrans);
    }
    AuditDataHelper.audit(Action.CREATE, Constants.GROUP ,
            "Group ${result.name} [id: ${result.id}] is added.", result);
    
    // if LDAP group then automatically populates group with users created in CB
    if (storage.rainmaker.security.enabled[]) {
        def dirContext = RainmakerLdapService.getInitialDirContext();
        def members = RainmakerLdapService.getGroupMembersDN(dirContext, group.name);
        if (members.size() > 0) {
            def users = getUsers([:]);
            users.each {
                if (it != null) {
                    def username = it.username;
                    def userid = it.id;
                    def userdn = RainmakerLdapService.findUserDnByUserName(dirContext, username);
                    if (userdn != null) {
                        if (members.find { it.equalsIgnoreCase(userdn) }) {
                            addGroupMember(result.id, userid);
                        }
                    }
                }
            }
        }	
        RainmakerLdapService.closeDirContext(dirContext);
    }


    fire('group-created', [group: group], false);
}

def deleteGroup(id)
{
    def groupDeleted = getGroup(id);
    def success = deleteResource('Groups', id);

    // Audit hook
    if(groupDeleted != null) {
    	def fieldsToTrans = [];
    	AuditDataHelper.transCodeToMessage(groupDeleted, fieldsToTrans);
    }
    AuditDataHelper.audit(Action.DELETE, Constants.GROUP,
            "Group ${groupDeleted.name} [id: ${id}] is deleted.", groupDeleted);

    getGroupRoles(id).each { deleteUserRole(user, it); }
    
    fire('group-deleted', [id: id], false);

    return success;
}

def getGroup(id)
{
    return getResource('Groups', [id: id]);
}

def getGroups(filters, owner = false)
{
    return getResources('Groups', filters, false);
}

def getGroupByName(name)
{
    return getResource('Groups', [name: name]);
}

def getGroupMembers(id)
{
   def ids = getResources('GroupsUsers', [groupid: id], false);
   return ids.collect { getUser(it.userid) };
}

def getUserGroups(user)
{
   def ids = getResources('GroupsUsers', [userid: user], false);
   return ids.collect { getGroup(it.groupid) };
}

def isUserInGroup(group, user)
{
    return getResource('GroupsUsers', [groupid: group, userid: user]) != null;
}

def removeGroupMember(group, user)
{
    def ret = deleteAllResources('GroupsUsers', [groupid: group, userid: user]);

    //Remove the roles from the user added by the group
    getUserRoles(user).each { deleteUserRole(user, it); } // remove all user roles
    getUserGroups(user).each { 
        getGroupRoles(it.id).each { addUserRole(user, it); } // add back roles for the remaining groups
    }
    return ret;
}

def updateGroup(group)
{
    def success = updateResource('Groups', group);    
    fire('group-updated', [group: group], false);

    // Audit hook
    def result = getGroup(group.id);
    if(result != null) {
    	def fieldsToTrans = [];
    	AuditDataHelper.transCodeToMessage(result, fieldsToTrans);
    }
    AuditDataHelper.audit(Action.UPDATE, Constants.GROUP,
            "Group ${result.name} [id: ${result.id}] is updated.", result);
    
    return success;
}

def getGroupRoles(id)
{
    def entries = getResources('GroupRoles', [groupid: id.toInteger()], false);
    return entries.collect { it.roleid };

}

def addGroupRole(id, role)
{

	createResource('GroupRoles', [groupid: id.toInteger(), roleid: role.toInteger()], false);
	
    // updating all group members and add this role to them
    getGroupMembers(id.toInteger()).each { user ->
        if (user) {
            addUserRole(user.id, role);
        }
    }
}

def deleteGroupRole(id, role)
{

	deleteAllResources('GroupRoles', [groupid: id.toInteger(), roleid: role.toInteger()]);
	
	// updating all group members and remove this role from them if
    // other user's groups don't have this role
    getGroupMembers(id.toInteger()).each { user ->
        if (user) {
            def gids = getResources('GroupsUsers', [userid: user.id], false);
            def doDeleteUserRole = true;
            for (gid in gids) {
                // check it is not EveryoneGroup and this group
                if (gid != null && gid.groupid != 1 && gid.groupid != id.toInteger()) {
                    def otherGroupRoles = getGroupRoles(gid.groupid);
                    if (otherGroupRoles.contains(role.toInteger())) {
                        doDeleteUserRole = false;
                        break;
                    }
                }
            }
            if (doDeleteUserRole) {
                deleteUserRole(user.id, role);
            }
        }
    }
    return true;
}

def isUserInMoreThanOneGroup(user)
{
    // check if user is in more than one group other than Everyone group
    def ids = getResources('GroupsUsers', [userid: user], false);
    if (ids != null && ids.size() > 0) {
        for (id in ids) {
            // check it is not EveryoneGroup
            if (id != null && id.groupid != 1) {
                return true;
            }
        }
    }
    return false;
}

def _getLanguage() {
    return user.lang[] ?: MessageUtils.getLanguageFromLocale(Locale.getDefault());
}
