/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.users;

import groovy.rainmaker.constants.*;
import java.util.List;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;
import com.ibm.websphere.rainmaker.auditing.AuditDataHelper;
import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.rainmaker.users.RainmakerLdapService;
import com.ibm.ws.xsa.common.Utility;
import zero.core.context.GlobalContext;

def addUserRole(id, role)
{
	def currentUserRoles = getUserRoles(id);
	if(currentUserRoles && currentUserRoles.contains(role)){
		logger.FINE { "user id ${id} already has role ${role}" }
		return;
	}else{
		createResource('UserRoles', [userid: id, roleid: role], false);
        fire('user-role-added',[user: id, role: role], false);
	}
}

def createAdmin()
{
    def lang = MessageUtils.getLanguageFromLocale(Locale.getDefault());
    
    def user = [
        username: MessageUtils.message(lang, 'RM02009', []), 
        name: MessageUtils.message(lang, 'RM02010', []), 
        email: ''
    ];
       
 //   inTransaction {
        try 
        {
            createUser(user, false, config.rainmaker.admin.defaultPassword[]);
	    	def encryptedPwd = zero.core.security.encryption.SecretToken.encryptString(config.security.collectiveSecretKey[], config.rainmaker.admin.defaultPassword[])
	    	zput(config.rainmaker.defaultPersistentZone[] + "/applianceUser",  encryptedPwd.toString()  )
        }
        catch (Throwable t) 
        {
            //
            // this is expected because the SMTP server 
            // is not configured yet
            //
            logger.FINE { CleanStackTrace.cleanStackTrace(t) };
        }
        
        logger.INFO { "user ${user}" }
        
        addUserRole(user.id, Permissions.SUPER_USER);
        addUserRole(user.id, Permissions.APPLIANCE_ADMIN);
        // do not add this role since it is done automatically for 
        // all users via the CreateRolesHandler for user-created event
        //addUserRole(user.id, Permissions.APPLIANCE_MONITOR);
        addUserRole(user.id, Permissions.APPLIANCE_SECURITY);
        addUserRole(user.id, Permissions.GRID_CREATE);
//    }
}

def createUser(user, owner = false, defaultPassword = false)
{
    def ldapUserDN = null;
    def internalGridUser = false;

    // if the username starts with the grid user prefix, we don't look in ldap
    String gridUserPrefix = GlobalContext.zget("/config/xsa/gridUserPrefix");
    if (gridUserPrefix != null  && user.username.startsWith(gridUserPrefix)) 
       internalGridUser = true;

    // if LDAP security is enabled, make sure is a valid LDAP user
    if (storage.rainmaker.security.enabled[] && !internalGridUser) {
        ldapUserDN = RainmakerLdapService.findUserDnByUserName(user.username);
        if (ldapUserDN == null) {
            throw new Exception(MessageUtils.message(_getLanguage(), 'RM02025', []));
        }
    }
	
    if (getUserByName(user.username))
        throw new Exception(MessageUtils.message(_getLanguage(), 'BR02004', [user.username]));
		
	List<String> invalidChars = Utility.getInvalidCharacters(user.username);
    if (invalidChars.size() > 0) {
        throw new Exception(MessageUtils.message(_getLanguage(), 'username.error.invalidName', invalidChars));
    }

    if (!Utility.validateNameLength(user.username)) {
        throw new Exception(MessageUtils.message(_getLanguage(), 'username.error.invalidNameLength', [String.valueOf(Utility.MAX_NAME_LENGTH)]));
    }
    
	// set attribute for non-system
	// users for filtering purposes in list
	if(!user.systemuser){
		user.systemuser = false;
	}
	
	def password = user.remove('password') ?: (defaultPassword ?: generatePassword(8));
	user.remove("password.verification");
    
	user.salt = generateSalt();
    user.passwordhash = hashPassword(password, user.salt);
    
    user.currentstatus = Status.LOGGED_OUT;
    user.currentmessage = 'RM02013';

    createResource('users', user, false);
    
    user.remove('salt');
    user.remove('passwordhash');
    
    fire('user-created', [user: user, password: password], false);
    
	// bug: 23271 dont send email if user is an LDAP user
    //if (!storage.rainmaker.security.enabled[] && internalGridUser)
    if (ldapUserDN == null)
		_sendNewUserEmail(user.username, password, user.email);

    // Audit hook
    def result = getUser(user.id);
    if(result != null) {
    	def fieldsToTrans = ["currentmessage", "currentstatus"];
    	AuditDataHelper.transCodeToMessage(result, fieldsToTrans);
    }
    AuditDataHelper.audit(Action.CREATE, Constants.USER ,
            "User ${result.name} [id: ${result.id}] is added.", result);
    
    // if LDAP user then automatically populates user with groups created in CB
    if (storage.rainmaker.security.enabled[] && !internalGridUser) {
        def groups = getGroups([:]);
        if (groups != null && groups.size() > 0) {
            def dirContext = RainmakerLdapService.getInitialDirContext();
            groups.each {
                if (it != null) {
                    def groupname = it.name;
                    def groupid = it.id;
                    if (groupid != 1) { // is not Everyone group
                        if (RainmakerLdapService.isUserDnInGroup(dirContext, ldapUserDN, groupname)) {
                            addGroupMember(groupid, result.id);
                        }
                    }
                }
            }
            RainmakerLdapService.closeDirContext(dirContext);
        }	
    }
    
}

def deleteUser(id)
{
    def userDeleted = getUser(id);
    def success = deleteResource('users', id);
    
    if (success)
        fire('user-deleted', [id: id,"username":userDeleted.username], false);
    
    // Audit hook
    if(userDeleted != null) {
    	def fieldsToTrans = ["currentmessage", "currentstatus"];
    	AuditDataHelper.transCodeToMessage(userDeleted, fieldsToTrans);
    }
    AuditDataHelper.audit(Action.DELETE, Constants.USER,
            "User ${userDeleted.name} [id: ${id}] is deleted.", userDeleted);

    return success;
}

def deleteUserRole(id, role)
{
	def userRoles = getUserRoles(id);
	if(userRoles && userRoles.contains(Permissions.SUPER_USER)){
		logger.FINER { "skip deleting user role for ${getUser(id)} since they are superUser" };
		def user = getUser(id);
	    AuditDataHelper.audit(Action.DELETE, Constants.USER,
	            "User ${user.name} [id: ${id}] is super user.  Deletion request ignored.", user);

		return true;
	}else{
		def retval = deleteAllResources('UserRoles', [userid: id, roleid: role]);
        if (retval)
            fire('user-role-deleted',[user: id, role: role], false);
        return retval;
	}
}

def getUser(id)
{
    return getResource('users', [id: id], ['id', 'username', 'name', 'email', 'currentstatus', 'currentmessage']);
}

def getUserByName(name)
{
	// if we know the system username, have to be able to return it for
	// various functions withing CB.
    return getUsers([username: name], false, true).find { it.username == name };
}

def getUsers(filters, owner = false, includeSystem = false)
{
	def _filters = filters.clone()
	// set attribute for non-system
	// users for filtering purposes in list
	// want to hide special system users
	if (!_filters.systemuser && !includeSystem){
		_filters.systemuser = false;
	}
    return getResources('users', _filters, false, ['id', 'username', 'name', 'email', 'currentstatus', 'currentmessage']);
}

def getUserRoles(id)
{
    def entries = getResources('UserRoles', [userid: id], false);
    return entries.collect { it.roleid };
}

def _sendNewUserEmail(userName, password, address)
{
    def subject = MessageUtils.message(user.lang[], 'RM02001', []);
    
    def content = """
${MessageUtils.message(user.lang[], 'BR02002', [])}

${MessageUtils.message(user.lang[], 'RM02003', [userName])}
${MessageUtils.message(user.lang[], 'RM02004', [password])}

${MessageUtils.message(user.lang[], 'RM02006', [request.serverName[]])}
""";
    
    try {
        if (!address.equals("")) {
            sendEmail(address, subject, content);
        }
    } catch (Throwable t) {
        logger.WARNING { MessageUtils.message(user.lang[], 'RM02005', [address]) };
    }
}

def updateUser(user)
{    
    def success = updateResource('users', user, ['name', 'email', 'currentstatus', 'currentmessage']);
    
    if (success)
        fire('user-updated', [user: user], false);

    // Audit hook
    
    // HACK: DO NOT audit the activities within last 5 minutes.
    if(user.currentmessage != 'RM02011') {
    	def result = getUser(user.id);
        if(result != null) {
        	def fieldsToTrans = ["currentmessage", "currentstatus"];
        	AuditDataHelper.transCodeToMessage(result, fieldsToTrans);
        }
        AuditDataHelper.audit(Action.UPDATE, Constants.USER,
                "User ${result.name} [id: ${result.id}] is updated.", result);
    }
    
    
    return success;
}

def _getLanguage() {
    return user.lang[] ?: MessageUtils.getLanguageFromLocale(Locale.getDefault());
}
